@@ -1,7 +1,11 @@
 import logging
 from collections import namedtuple
-from typing import Dict, Any
+from typing import (
+    Any,
+    Dict,
+)
 
+from pcs import settings
 from pcs.cli.common import middleware
 from pcs.lib.commands import (
     acl,
@@ -9,6 +13,7 @@ from pcs.lib.commands import (
     booth,
     cib_options,
     cluster,
+    cluster_property,
     dr,
     fencing_topology,
     node,
@@ -19,21 +24,18 @@ from pcs.lib.commands import (
     resource,
     resource_agent,
     sbd,
-    services,
     scsi,
+    services,
     status,
     stonith,
     stonith_agent,
     tag,
 )
-from pcs.lib.commands.constraint import (
-    colocation as constraint_colocation,
-    order as constraint_order,
-    ticket as constraint_ticket,
-)
+from pcs.lib.commands.constraint import colocation as constraint_colocation
+from pcs.lib.commands.constraint import order as constraint_order
+from pcs.lib.commands.constraint import ticket as constraint_ticket
 from pcs.lib.env import LibraryEnvironment
 
-
 # Note: not properly typed
 _CACHE: Dict[Any, Any] = {}
 
@@ -151,30 +153,39 @@ def load_module(env, middleware_factory, name):
         )
 
     if name == "booth":
+        bindings = {
+            "config_setup": booth.config_setup,
+            "config_destroy": booth.config_destroy,
+            "config_text": booth.config_text,
+            "config_ticket_add": booth.config_ticket_add,
+            "config_ticket_remove": booth.config_ticket_remove,
+            "create_in_cluster": booth.create_in_cluster,
+            "remove_from_cluster": booth.remove_from_cluster,
+            "restart": booth.restart,
+            "config_sync": booth.config_sync,
+            "enable_booth": booth.enable_booth,
+            "disable_booth": booth.disable_booth,
+            "start_booth": booth.start_booth,
+            "stop_booth": booth.stop_booth,
+            "pull_config": booth.pull_config,
+            "get_status": booth.get_status,
+            "ticket_grant": booth.ticket_grant,
+            "ticket_revoke": booth.ticket_revoke,
+        }
+        if settings.booth_enable_authfile_set_enabled:
+            bindings[
+                "config_set_enable_authfile"
+            ] = booth.config_set_enable_authfile
+        if settings.booth_enable_authfile_unset_enabled:
+            bindings[
+                "config_unset_enable_authfile"
+            ] = booth.config_unset_enable_authfile
         return bind_all(
             env,
             middleware.build(
                 middleware_factory.booth_conf, middleware_factory.cib
             ),
-            {
-                "config_setup": booth.config_setup,
-                "config_destroy": booth.config_destroy,
-                "config_text": booth.config_text,
-                "config_ticket_add": booth.config_ticket_add,
-                "config_ticket_remove": booth.config_ticket_remove,
-                "create_in_cluster": booth.create_in_cluster,
-                "remove_from_cluster": booth.remove_from_cluster,
-                "restart": booth.restart,
-                "config_sync": booth.config_sync,
-                "enable_booth": booth.enable_booth,
-                "disable_booth": booth.disable_booth,
-                "start_booth": booth.start_booth,
-                "stop_booth": booth.stop_booth,
-                "pull_config": booth.pull_config,
-                "get_status": booth.get_status,
-                "ticket_grant": booth.ticket_grant,
-                "ticket_revoke": booth.ticket_revoke,
-            },
+            bindings,
         )
 
     if name == "cluster":
@@ -196,6 +207,8 @@ def load_module(env, middleware_factory, name):
                 "setup_local": cluster.setup_local,
                 "update_link": cluster.update_link,
                 "verify": cluster.verify,
+                "generate_cluster_uuid": cluster.generate_cluster_uuid,
+                "generate_cluster_uuid_local": cluster.generate_cluster_uuid_local,
             },
         )
 
@@ -348,10 +361,14 @@ def load_module(env, middleware_factory, name):
             middleware.build(),
             {
                 "describe_agent": resource_agent.describe_agent,
-                "list_agents": resource_agent.list_agents,
+                "get_agent_default_operations": resource_agent.get_agent_default_operations,
+                "get_agent_metadata": resource_agent.get_agent_metadata,
+                "get_agents_list": resource_agent.get_agents_list,
+                "get_structured_agent_name": resource_agent.get_structured_agent_name,
                 "list_agents_for_standard_and_provider": (
                     resource_agent.list_agents_for_standard_and_provider
                 ),
+                "list_agents": resource_agent.list_agents,
                 "list_ocf_providers": resource_agent.list_ocf_providers,
                 "list_standards": resource_agent.list_standards,
             },
@@ -377,8 +394,11 @@ def load_module(env, middleware_factory, name):
                 "disable_safe": resource.disable_safe,
                 "disable_simulate": resource.disable_simulate,
                 "enable": resource.enable,
+                "get_configured_resources": resource.get_configured_resources,
                 "get_failcounts": resource.get_failcounts,
                 "group_add": resource.group_add,
+                "is_any_resource_except_stonith": resource.is_any_resource_except_stonith,
+                "is_any_stonith": resource.is_any_stonith,
                 "manage": resource.manage,
                 "move": resource.move,
                 "move_autoclean": resource.move_autoclean,
@@ -416,6 +436,7 @@ def load_module(env, middleware_factory, name):
                 middleware_factory.corosync_conf_existing,
             ),
             {
+                "pacemaker_status_xml": status.pacemaker_status_xml,
                 "full_cluster_status_plaintext": (
                     status.full_cluster_status_plaintext
                 ),
@@ -478,6 +499,7 @@ def load_module(env, middleware_factory, name):
             middleware.build(),
             {
                 "unfence_node": scsi.unfence_node,
+                "unfence_node_mpath": scsi.unfence_node_mpath,
             },
         )
 
@@ -503,6 +525,15 @@ def load_module(env, middleware_factory, name):
             },
         )
 
+    if name == "cluster_property":
+        return bind_all(
+            env,
+            middleware.build(middleware_factory.cib),
+            {
+                "set_properties": cluster_property.set_properties,
+            },
+        )
+
     raise Exception("No library part '{0}'".format(name))
 
 
