@@ -1,20 +1,30 @@
+from collections import Counter
+from collections.abc import Set
 from typing import (
-    Mapping,
     Iterable,
+    Mapping,
     Union,
+    cast,
 )
 
 from pcs.cli.common.errors import (
+    SEE_MAN_CHANGES,
     CmdLineInputError,
-    HINT_SYNTAX_CHANGE,
 )
 from pcs.common.str_tools import (
     format_list,
     format_plural,
 )
+from pcs.common.tools import timeout_to_seconds
+from pcs.common.types import StringSequence
 
 ModifierValueType = Union[None, bool, str]
 
+_OUTPUT_FORMAT_OPTION_STR = "output-format"
+_OUTPUT_FORMAT_OPTION = f"--{_OUTPUT_FORMAT_OPTION_STR}"
+OUTPUT_FORMAT_VALUE_TEXT = "text"
+_OUTPUT_FORMAT_VALUES = frozenset((OUTPUT_FORMAT_VALUE_TEXT, "cmd", "json"))
+
 ARG_TYPE_DELIMITER = "%"
 
 # h = help, f = file,
@@ -27,8 +37,6 @@ PCS_LONG_OPTIONS = [
     "fullhelp",
     "force",
     "skip-offline",
-    # TODO remove, deprecated command 'pcs config import-cman'
-    "interactive",
     "autodelete",
     "simulate",
     "all",
@@ -52,7 +60,8 @@ PCS_LONG_OPTIONS = [
     "no-default-ops",
     "defaults",
     "nodesc",
-    "master",
+    "master",  # deprecated, replaced by --promoted
+    "promoted",
     "name=",
     "group=",
     "node=",
@@ -64,6 +73,8 @@ PCS_LONG_OPTIONS = [
     "booth-conf=",
     "booth-key=",
     "no-watchdog-validation",
+    # pcs cluster setup
+    "no-cluster-uuid",
     "no-keys-sync",
     # in pcs status - do not display resource status on inactive node
     "hide-inactive",
@@ -79,7 +90,7 @@ PCS_LONG_OPTIONS = [
     # allow overwriting existing files, currently meant for / used in CLI only
     "overwrite",
     # output format of commands, e.g: json, cmd, text, ...
-    "output-format=",
+    f"{_OUTPUT_FORMAT_OPTION_STR}=",
     # auth token
     "token=",
 ]
@@ -130,7 +141,7 @@ def split_option(arg, allow_empty_value=True):
     Get (key, value) from a key=value commandline argument.
 
     Split the argument by the first = and return resulting parts. Raise
-    CmdLineInputError if the argument cannot be splitted.
+    CmdLineInputError if the argument cannot be split.
 
     string arg -- commandline argument
     allow_empty_value -- if True, empty value is allowed. Otherwise,
@@ -148,6 +159,19 @@ def split_option(arg, allow_empty_value=True):
     return key, value
 
 
+def ensure_unique_args(cmdline_args: StringSequence) -> None:
+    """
+    Raises in case there are duplicate args
+    """
+    duplicities = [
+        item for item, count in Counter(cmdline_args).items() if count > 1
+    ]
+    if duplicities:
+        argument_pl = format_plural(duplicities, "argument")
+        duplicities_list = format_list(duplicities)
+        raise CmdLineInputError(f"duplicate {argument_pl}: {duplicities_list}")
+
+
 def prepare_options(cmdline_args, allowed_repeatable_options=()):
     """
     Get a dict of options from cmdline key=value args
@@ -157,7 +181,7 @@ def prepare_options(cmdline_args, allowed_repeatable_options=()):
 
     Commandline options: no options
     """
-    options = dict()
+    options = {}
     for arg in cmdline_args:
         name, value = split_option(arg)
         if name not in options:
@@ -222,10 +246,10 @@ def group_by_keywords(
     set keyword_set contain all expected keywords
     string implicit_first_group_key is the key for capturing of arguments before
         the occurrence of the first keyword. implicit_first_group_key is not
-        a keyword => its occurence in args is considered as ordinary argument.
+        a keyword => its occurrence in args is considered as ordinary argument.
     bool keyword_repeat_allowed is the flag to turn on/off checking the
         uniqueness of each keyword in arg_list.
-    list group_repeated_keywords contains keywords for which each occurence is
+    list group_repeated_keywords contains keywords for which each occurrence is
         packed separately. For example when keywords are "first" and "seconds"
         and group_repeated_keywords is ["first"] then for arg_list
         ["first", 1, 2, "second", 3, "first", 4] it returns
@@ -264,15 +288,15 @@ def group_by_keywords(
                 completed_groups[implicit_first_group_key] = []
         return completed_groups
 
-    def is_acceptable_keyword_occurence(keyword):
+    def is_acceptable_keyword_occurrence(keyword):
         return (
-            keyword not in groups.keys()
+            keyword not in groups
             or keyword_repeat_allowed
             or keyword in keywords_for_grouping
         )
 
     def process_keyword(keyword):
-        if not is_acceptable_keyword_occurence(keyword):
+        if not is_acceptable_keyword_occurrence(keyword):
             raise CmdLineInputError(
                 "'{0}' cannot be used more than once".format(keyword)
             )
@@ -441,6 +465,17 @@ def filter_out_options(arg_list):
     return args_without_options
 
 
+def wait_to_timeout(wait: Union[bool, str, None]) -> int:
+    if wait is False:
+        return -1
+    if wait is None:
+        return 0
+    timeout = timeout_to_seconds(wait)
+    if timeout is None:
+        raise CmdLineInputError(f"'{wait}' is not a valid interval value")
+    return timeout
+
+
 class InputModifiers:
     def __init__(self, options: Mapping[str, ModifierValueType]):
         self._defined_options = set(options.keys())
@@ -464,14 +499,12 @@ class InputModifiers:
                 # used only in deprecated 'pcs resource|stonith show'
                 "--groups": "--groups" in options,
                 "--hide-inactive": "--hide-inactive" in options,
-                # TODO remove, deprecated command 'pcs config import-cman'
-                "--interactive": "--interactive" in options,
                 "--local": "--local" in options,
-                "--master": "--master" in options,
                 "--monitor": "--monitor" in options,
                 "--no-default-ops": "--no-default-ops" in options,
                 "--nodesc": "--nodesc" in options,
                 "--no-expire-check": "--no-expire-check" in options,
+                "--no-cluster-uuid": "--no-cluster-uuid" in options,
                 "--no-keys-sync": "--no-keys-sync" in options,
                 "--no-strict": "--no-strict" in options,
                 "--no-watchdog-validation": "--no-watchdog-validation"
@@ -479,6 +512,7 @@ class InputModifiers:
                 "--off": "--off" in options,
                 "--overwrite": "--overwrite" in options,
                 "--pacemaker": "--pacemaker" in options,
+                "--promoted": "--promoted" in options,
                 "--safe": "--safe" in options,
                 "--simulate": "--simulate" in options,
                 "--skip-offline": "--skip-offline" in options,
@@ -494,7 +528,9 @@ class InputModifiers:
                 "--group": options.get("--group", None),
                 "--name": options.get("--name", None),
                 "--node": options.get("--node", None),
-                "--output-format": options.get("--output-format", "text"),
+                _OUTPUT_FORMAT_OPTION: options.get(
+                    _OUTPUT_FORMAT_OPTION, OUTPUT_FORMAT_VALUE_TEXT
+                ),
                 "--request-timeout": options.get("--request-timeout", None),
                 "--to": options.get("--to", None),
                 "--token": options.get("--token", None),
@@ -513,14 +549,16 @@ class InputModifiers:
         return InputModifiers(opt_dict)
 
     def ensure_only_supported(
-        self, *supported_options, hint_syntax_changed: bool = False
-    ):
-        unsupported_options = (
-            # --debug is supported in all commands
-            self._defined_options
-            - set(supported_options)
-            - set(["--debug"])
-        )
+        self,
+        *supported_options: str,
+        hint_syntax_changed: bool = False,
+        output_format_supported: bool = False,
+    ) -> None:
+        # --debug is supported in all commands
+        supported_options_set = set(supported_options) | {"--debug"}
+        if output_format_supported:
+            supported_options_set.add(_OUTPUT_FORMAT_OPTION)
+        unsupported_options = self._defined_options - supported_options_set
         if unsupported_options:
             pluralize = lambda word: format_plural(unsupported_options, word)
             raise CmdLineInputError(
@@ -530,10 +568,11 @@ class InputModifiers:
                     option_list=format_list(sorted(unsupported_options)),
                     _is=pluralize("is"),
                 ),
-                # Print error messages which point users to the changes section
-                # in pcs manpage.
-                # To be removed in the next significant version.
-                hint=(HINT_SYNTAX_CHANGE if hint_syntax_changed else None),
+                hint="Syntax has changed from previous version. {}".format(
+                    SEE_MAN_CHANGES.format("0.11")
+                )
+                if hint_syntax_changed
+                else None,
             )
 
     def ensure_not_mutually_exclusive(self, *mutually_exclusive):
@@ -556,6 +595,29 @@ class InputModifiers:
                 )
             )
 
+    def ensure_dependency_satisfied(
+        self,
+        main_option: str,
+        dependent_options: Iterable[str],
+    ) -> None:
+        """
+        Raise a `CmdLineInputError` exception if any of `dependent_options` is
+        present without `main_option` being present.
+
+        main_option -- option on which `dependent_options` depend
+        dependent_options -- none of these options can be specified if
+            `main_option` was not
+        """
+        if main_option in self._defined_options:
+            return
+        disallowed = self._defined_options & set(dependent_options)
+        if disallowed:
+            raise CmdLineInputError(
+                "{} cannot be used without '{}'".format(
+                    format_list(sorted(disallowed)), main_option
+                )
+            )
+
     def is_specified(self, option: str) -> bool:
         return option in self._defined_options
 
@@ -575,3 +637,22 @@ class InputModifiers:
         if option in self._options:
             return self._options[option]
         raise AssertionError(f"Non existing default value for '{option}'")
+
+    def get_output_format(
+        self, supported_formats: Set[str] = _OUTPUT_FORMAT_VALUES
+    ) -> str:
+        output_format = self.get(_OUTPUT_FORMAT_OPTION)
+        if output_format in supported_formats:
+            return cast(str, output_format)
+        raise CmdLineInputError(
+            (
+                "Unknown value '{value}' for '{option}' option. Supported "
+                "{value_pl} {is_pl}: {supported}"
+            ).format(
+                value=output_format,
+                option=_OUTPUT_FORMAT_OPTION,
+                value_pl=format_plural(supported_formats, "value"),
+                is_pl=format_plural(supported_formats, "is"),
+                supported=format_list(list(supported_formats)),
+            )
+        )
