@@ -1,3 +1,9 @@
+from typing import (
+    Any,
+    Sequence,
+)
+
+import pcs.cli.cluster.command as cluster_command
 from pcs import (
     cluster,
     pcsd,
@@ -5,19 +11,51 @@ from pcs import (
     status,
     usage,
 )
-import pcs.cli.cluster.command as cluster_command
-from pcs.cli.common.errors import raise_command_replaced
+from pcs.cli.common.errors import CmdLineInputError
+from pcs.cli.common.parse_args import InputModifiers
 from pcs.cli.common.routing import create_router
+from pcs.cli.reports.output import deprecation_warning
+from pcs.utils import exit_on_cmdline_input_error
+
+
+def certkey(lib: Any, argv: Sequence[str], modifiers: InputModifiers) -> None:
+    deprecation_warning(
+        "This command is deprecated and will be removed. "
+        "Please use 'pcs pcsd certkey' instead."
+    )
+    try:
+        return pcsd.pcsd_certkey_cmd(lib, argv, modifiers)
+    except CmdLineInputError as e:
+        return exit_on_cmdline_input_error(e, "pcsd", ["certkey"])
+
+
+def pcsd_status(
+    lib: Any, argv: Sequence[str], modifiers: InputModifiers
+) -> None:
+    deprecation_warning(
+        "This command is deprecated and will be removed. "
+        "Please use 'pcs pcsd status' or 'pcs status pcsd' instead."
+    )
+    try:
+        return pcsd.pcsd_status_cmd(lib, argv, modifiers)
+    except CmdLineInputError as e:
+        return exit_on_cmdline_input_error(e, "pcsd", ["status"])
 
 
 cluster_cmd = create_router(
     {
-        "help": lambda lib, argv, modifiers: usage.cluster(argv),
+        "help": lambda lib, argv, modifiers: print(usage.cluster(argv)),
         "setup": cluster.cluster_setup,
         "config": create_router(
             {
                 "show": cluster.config_show,
                 "update": cluster.config_update,
+                "uuid": create_router(
+                    {
+                        "generate": cluster.generate_uuid,
+                    },
+                    ["cluster", "config", "uuid"],
+                ),
             },
             ["cluster", "config"],
             default_cmd="show",
@@ -34,8 +72,12 @@ cluster_cmd = create_router(
             default_cmd="corosync",
         ),
         "status": status.cluster_status,
-        "pcsd-status": status.cluster_pcsd_status,
-        "certkey": pcsd.pcsd_certkey,
+        # TODO remove, deprecated command
+        # replaced with 'pcs pcsd status' and 'pcs status pcsd'
+        "pcsd-status": pcsd_status,
+        # TODO remove, deprecated command
+        # replaced with 'pcs pcsd certkey'
+        "certkey": certkey,
         "auth": cluster.cluster_auth_cmd,
         "start": cluster.cluster_start_cmd,
         "stop": cluster.cluster_stop_cmd,
@@ -82,31 +124,6 @@ cluster_cmd = create_router(
         "verify": cluster.cluster_verify,
         "report": cluster.cluster_report,
         "remove_nodes_from_cib": cluster.remove_nodes_from_cib,
-        # removed commands
-        # These print error messages which point users to the changes section in
-        # pcs manpage.
-        # To be removed in the next significant version.
-        "quorum": lambda lib, argv, modifiers: raise_command_replaced(
-            "pcs quorum"
-        ),
-        "remote-node": create_router(
-            {
-                "add": lambda lib, argv, modifiers: raise_command_replaced(
-                    "pcs cluster node add-guest",
-                ),
-                "remove": lambda lib, argv, modifiers: raise_command_replaced(
-                    "pcs cluster node delete-guest",
-                    "pcs cluster node remove-guest",
-                ),
-            },
-            ["cluster", "node"],
-        ),
-        "standby": lambda lib, argv, modifiers: raise_command_replaced(
-            "pcs node standby"
-        ),
-        "unstandby": lambda lib, argv, modifiers: raise_command_replaced(
-            "pcs node unstandby"
-        ),
     },
     ["cluster"],
 )
