@@ -2,9 +2,16 @@ from typing import (
     Any,
     Sequence,
 )
+
 from pcs.cli.common.errors import CmdLineInputError
-from pcs.cli.common.parse_args import group_by_keywords, InputModifiers
-from pcs.cli.reports.output import warn
+from pcs.cli.common.parse_args import (
+    InputModifiers,
+    group_by_keywords,
+)
+from pcs.cli.reports.output import (
+    deprecation_warning,
+    print_to_stderr,
+)
 from pcs.common.str_tools import indent
 
 
@@ -33,10 +40,9 @@ def tag_list_cmd(
     Options:
       * -f - CIB file
     """
-    warn(
+    deprecation_warning(
         "This command is deprecated and will be removed. "
-        "Please use 'pcs tag config' instead.",
-        stderr=True,
+        "Please use 'pcs tag config' instead."
     )
     return tag_config(lib, argv, modifiers)
 
@@ -53,7 +59,7 @@ def tag_config(
     modifiers.ensure_only_supported("-f")
     tag_list = lib.tag.config(argv)
     if not tag_list:
-        print(" No tags defined")
+        print_to_stderr(" No tags defined")
         return
     lines = []
     for tag in tag_list:
