@@ -1,3 +1,7 @@
+# pylint: disable=wildcard-import, unused-wildcard-import
+# Wildcard import of deprecated report codes will prevent creation of a new
+# report with the code of deprecated report
+from .deprecated_codes import *
 from .types import ForceCode as F
 from .types import MessageCode as M
 
@@ -39,9 +43,15 @@ AGENT_IMPLEMENTS_UNSUPPORTED_OCF_VERSION = M(
 AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE = M("AGENT_NAME_GUESS_FOUND_MORE_THAN_ONE")
 AGENT_NAME_GUESS_FOUND_NONE = M("AGENT_NAME_GUESS_FOUND_NONE")
 AGENT_NAME_GUESSED = M("AGENT_NAME_GUESSED")
+AGENT_SELF_VALIDATION_INVALID_DATA = M("AGENT_SELF_VALIDATION_INVALID_DATA")
+AGENT_SELF_VALIDATION_RESULT = M("AGENT_SELF_VALIDATION_RESULT")
 BAD_CLUSTER_STATE_FORMAT = M("BAD_CLUSTER_STATE_FORMAT")
 BOOTH_ADDRESS_DUPLICATION = M("BOOTH_ADDRESS_DUPLICATION")
 BOOTH_ALREADY_IN_CIB = M("BOOTH_ALREADY_IN_CIB")
+BOOTH_AUTHFILE_NOT_USED = M("BOOTH_AUTHFILE_NOT_USED")
+BOOTH_UNSUPPORTED_OPTION_ENABLE_AUTHFILE = M(
+    "BOOTH_UNSUPPORTED_OPTION_ENABLE_AUTHFILE"
+)
 BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP = M("BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP")
 BOOTH_CONFIG_ACCEPTED_BY_NODE = M("BOOTH_CONFIG_ACCEPTED_BY_NODE")
 BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR = M("BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR")
@@ -69,24 +79,9 @@ CANNOT_BAN_RESOURCE_MASTER_RESOURCE_NOT_PROMOTABLE = M(
 CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED = M(
     "CANNOT_BAN_RESOURCE_STOPPED_NO_NODE_SPECIFIED"
 )
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP = M(
-    "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_FOR_NEW_GROUP"
+CANNOT_CREATE_DEFAULT_CLUSTER_PROPERTY_SET = M(
+    "CANNOT_CREATE_DEFAULT_CLUSTER_PROPERTY_SET"
 )
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP = M(
-    "CANNOT_GROUP_RESOURCE_ADJACENT_RESOURCE_NOT_IN_GROUP"
-)
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP = M(
-    "CANNOT_GROUP_RESOURCE_ALREADY_IN_THE_GROUP"
-)
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE = M("CANNOT_GROUP_RESOURCE_MORE_THAN_ONCE")
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF = M("CANNOT_GROUP_RESOURCE_NEXT_TO_ITSELF")
-# TODO: remove, use ADD_REMOVE reports
-CANNOT_GROUP_RESOURCE_NO_RESOURCES = M("CANNOT_GROUP_RESOURCE_NO_RESOURCES")
 CANNOT_GROUP_RESOURCE_WRONG_TYPE = M("CANNOT_GROUP_RESOURCE_WRONG_TYPE")
 CANNOT_LEAVE_GROUP_EMPTY_AFTER_MOVE = M("CANNOT_LEAVE_GROUP_EMPTY_AFTER_MOVE")
 CANNOT_MOVE_RESOURCE_BUNDLE = M("CANNOT_MOVE_RESOURCE_BUNDLE")
@@ -148,6 +143,8 @@ CLUSTER_START_STARTED = M("CLUSTER_START_STARTED")
 CLUSTER_START_SUCCESS = M("CLUSTER_START_SUCCESS")
 CLUSTER_UUID_ALREADY_SET = M("CLUSTER_UUID_ALREADY_SET")
 CLUSTER_WILL_BE_DESTROYED = M("CLUSTER_WILL_BE_DESTROYED")
+COMMAND_INVALID_PAYLOAD = M("COMMAND_INVALID_PAYLOAD")
+COMMAND_UNKNOWN = M("COMMAND_UNKNOWN")
 LIVE_ENVIRONMENT_NOT_CONSISTENT = M("LIVE_ENVIRONMENT_NOT_CONSISTENT")
 LIVE_ENVIRONMENT_REQUIRED = M("LIVE_ENVIRONMENT_REQUIRED")
 LIVE_ENVIRONMENT_REQUIRED_FOR_LOCAL_NODE = M(
@@ -234,8 +231,6 @@ COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS = M(
     "COROSYNC_TRANSPORT_UNSUPPORTED_OPTIONS"
 )
 CRM_MON_ERROR = M("CRM_MON_ERROR")
-# TODO remove
-DEFAULTS_CAN_BE_OVERRIDEN = M("DEFAULTS_CAN_BE_OVERRIDEN")
 DEFAULTS_CAN_BE_OVERRIDDEN = M("DEFAULTS_CAN_BE_OVERRIDDEN")
 DEPRECATED_OPTION = M("DEPRECATED_OPTION")
 DEPRECATED_OPTION_VALUE = M("DEPRECATED_OPTION_VALUE")
@@ -275,7 +270,6 @@ INVALID_RESPONSE_FORMAT = M("INVALID_RESPONSE_FORMAT")
 INVALID_SCORE = M("INVALID_SCORE")
 INVALID_STONITH_AGENT_NAME = M("INVALID_STONITH_AGENT_NAME")
 INVALID_TIMEOUT_VALUE = M("INVALID_TIMEOUT_VALUE")
-MULTIPLE_SCORE_OPTIONS = M("MULTIPLE_SCORE_OPTIONS")
 MULTIPLE_RESULTS_FOUND = M("MULTIPLE_RESULTS_FOUND")
 MUTUALLY_EXCLUSIVE_OPTIONS = M("MUTUALLY_EXCLUSIVE_OPTIONS")
 NO_ACTION_NECESSARY = M("NO_ACTION_NECESSARY")
@@ -316,8 +310,9 @@ NODE_USED_AS_TIE_BREAKER = M("NODE_USED_AS_TIE_BREAKER")
 NODES_TO_REMOVE_UNREACHABLE = M("NODES_TO_REMOVE_UNREACHABLE")
 NODE_TO_CLEAR_IS_STILL_IN_CLUSTER = M("NODE_TO_CLEAR_IS_STILL_IN_CLUSTER")
 NODE_IN_LOCAL_CLUSTER = M("NODE_IN_LOCAL_CLUSTER")
-OMITTING_NODE = M("OMITTING_NODE")
+NOT_AUTHORIZED = M("NOT_AUTHORIZED")
 OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT = M("OBJECT_WITH_ID_IN_UNEXPECTED_CONTEXT")
+OMITTING_NODE = M("OMITTING_NODE")
 PACEMAKER_SIMULATION_RESULT = M("PACEMAKER_SIMULATION_RESULT")
 PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND = M("PACEMAKER_LOCAL_NODE_NAME_NOT_FOUND")
 PARSE_ERROR_COROSYNC_CONF = M("PARSE_ERROR_COROSYNC_CONF")
@@ -392,6 +387,9 @@ RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE = M(
     "RESOURCE_BUNDLE_UNSUPPORTED_CONTAINER_TYPE"
 )
 RESOURCE_CLEANUP_ERROR = M("RESOURCE_CLEANUP_ERROR")
+RESOURCE_CLONE_INCOMPATIBLE_META_ATTRIBUTES = M(
+    "RESOURCE_CLONE_INCOMPATIBLE_META_ATTRIBUTES"
+)
 RESOURCE_DOES_NOT_RUN = M("RESOURCE_DOES_NOT_RUN")
 RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES = M(
     "RESOURCE_DISABLE_AFFECTS_OTHER_RESOURCES"
@@ -460,7 +458,6 @@ SBD_DEVICE_MESSAGE_ERROR = M("SBD_DEVICE_MESSAGE_ERROR")
 SBD_DEVICE_PATH_NOT_ABSOLUTE = M("SBD_DEVICE_PATH_NOT_ABSOLUTE")
 SBD_LIST_WATCHDOG_ERROR = M("SBD_LIST_WATCHDOG_ERROR")
 SBD_NO_DEVICE_FOR_NODE = M("SBD_NO_DEVICE_FOR_NODE")
-SBD_NOT_INSTALLED = M("SBD_NOT_INSTALLED")
 SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS = M("SBD_NOT_USED_CANNOT_SET_SBD_OPTIONS")
 SBD_TOO_MANY_DEVICES_FOR_NODE = M("SBD_TOO_MANY_DEVICES_FOR_NODE")
 SBD_WITH_DEVICES_NOT_USED_CANNOT_SET_DEVICE = M(
@@ -562,8 +559,6 @@ UNABLE_TO_CONNECT_TO_ANY_REMAINING_NODE = M(
 UNABLE_TO_CONNECT_TO_ALL_REMAINING_NODE = M(
     "UNABLE_TO_CONNECT_TO_ALL_REMAINING_NODE"
 )
-UNABLE_TO_DETERMINE_USER_UID = M("UNABLE_TO_DETERMINE_USER_UID")
-UNABLE_TO_DETERMINE_GROUP_GID = M("UNABLE_TO_DETERMINE_GROUP_GID")
 UNABLE_TO_GET_AGENT_METADATA = M("UNABLE_TO_GET_AGENT_METADATA")
 UNABLE_TO_GET_SBD_CONFIG = M("UNABLE_TO_GET_SBD_CONFIG")
 UNABLE_TO_GET_SBD_STATUS = M("UNABLE_TO_GET_SBD_STATUS")
