@@ -1,29 +1,22 @@
-from __future__ import (
-    absolute_import,
-    division,
-    print_function,
-)
-
 from functools import partial
 
 from lxml import etree
 
-from pcs.lib import reports
-from pcs.lib.errors import LibraryError
+from pcs.common import reports
 from pcs.lib.cib.tools import (
     check_new_id_applicable,
     does_id_exist,
-    find_unique_id,
     find_element_by_tag_and_id,
+    find_unique_id,
 )
-from pcs.lib.xml_tools import etree_element_attibutes_to_dict
-
+from pcs.lib.errors import LibraryError
 
 TAG_GROUP = "acl_group"
 TAG_ROLE = "acl_role"
 TAG_TARGET = "acl_target"
 TAG_PERMISSION = "acl_permission"
 
+
 def validate_permissions(tree, permission_info_list):
     """
     Validate given permission list.
@@ -38,29 +31,35 @@ def validate_permissions(tree, permission_info_list):
     allowed_scopes = ["xpath", "id"]
     for permission, scope_type, scope in permission_info_list:
         if not permission in allowed_permissions:
-            report_items.append(reports.invalid_option_value(
-                "permission",
-                permission,
-                allowed_permissions
-            ))
+            report_items.append(
+                reports.ReportItem.error(
+                    reports.messages.InvalidOptionValue(
+                        "permission", permission, allowed_permissions
+                    )
+                )
+            )
 
         if not scope_type in allowed_scopes:
-            report_items.append(reports.invalid_option_value(
-                "scope type",
-                scope_type,
-                allowed_scopes
-            ))
+            report_items.append(
+                reports.ReportItem.error(
+                    reports.messages.InvalidOptionValue(
+                        "scope type", scope_type, allowed_scopes
+                    )
+                )
+            )
 
-        if scope_type == 'id' and not does_id_exist(tree, scope):
-            report_items.append(reports.id_not_found(scope, ["id"]))
+        if scope_type == "id" and not does_id_exist(tree, scope):
+            report_items.append(
+                reports.ReportItem.error(
+                    reports.messages.IdNotFound(scope, ["id"])
+                )
+            )
 
     if report_items:
         raise LibraryError(*report_items)
 
 
-def _find(
-    tag, acl_section, element_id, none_if_id_unused=False, id_types=None
-):
+def _find(tag, acl_section, element_id, none_if_id_unused=False, id_types=None):
     return find_element_by_tag_and_id(
         tag,
         acl_section,
@@ -69,10 +68,15 @@ def _find(
         none_if_id_unused=none_if_id_unused,
     )
 
+
 find_group = partial(_find, TAG_GROUP)
 find_role = partial(_find, TAG_ROLE)
 find_target = partial(_find, TAG_TARGET)
 
+
+# TODO deprecate
+# We haven't deprecated it yet, as groups don't work in pacemaker, therefore
+# there would be no benefit from deprecating it.
 def find_target_or_group(acl_section, target_or_group_id):
     """
     Returns acl_target or acl_group element with id target_or_group_id. Target
@@ -89,20 +93,17 @@ def find_target_or_group(acl_section, target_or_group_id):
     target_or_group_id -- id of target/group element which should be returned
     """
     target = find_target(
-        acl_section,
-        target_or_group_id,
-        none_if_id_unused=True
+        acl_section, target_or_group_id, none_if_id_unused=True
     )
 
     if target is not None:
         return target
 
     return find_group(
-        acl_section,
-        target_or_group_id,
-        id_types=[TAG_GROUP, TAG_TARGET]
+        acl_section, target_or_group_id, id_types=[TAG_GROUP, TAG_TARGET]
     )
 
+
 def create_role(acl_section, role_id, description=None):
     """
     Create new role element and add it to cib.
@@ -128,24 +129,29 @@ def remove_role(acl_section, role_id, autodelete_users_groups=False):
     """
     acl_role = find_role(acl_section, role_id)
     acl_role.getparent().remove(acl_role)
-    for role_el in acl_section.findall(".//role[@id='{0}']".format(role_id)):
+    for role_el in acl_section.xpath(".//role[@id=$role_id]", role_id=role_id):
         role_parent = role_el.getparent()
         role_parent.remove(role_el)
         if autodelete_users_groups and role_parent.find(".//role") is None:
             role_parent.getparent().remove(role_parent)
 
+
 def _assign_role(acl_section, role_id, target_el):
     try:
         role_el = find_role(acl_section, role_id)
     except LibraryError as e:
         return list(e.args)
-    assigned_role = target_el.find(
-        "./role[@id='{0}']".format(role_el.get("id"))
+    assigned_role_list = target_el.xpath(
+        "./role[@id=$role_id]", role_id=role_el.get("id")
     )
-    if assigned_role is not None:
-        return [reports.acl_role_is_already_assigned_to_target(
-            role_el.get("id"), target_el.get("id")
-        )]
+    if assigned_role_list:
+        return [
+            reports.ReportItem.error(
+                reports.messages.CibAclRoleIsAlreadyAssignedToTarget(
+                    role_el.get("id"), target_el.get("id")
+                )
+            )
+        ]
     etree.SubElement(target_el, "role", {"id": role_el.get("id")})
     return []
 
@@ -162,6 +168,7 @@ def assign_role(acl_section, role_id, target_el):
     if report_list:
         raise LibraryError(*report_list)
 
+
 def assign_all_roles(acl_section, role_id_list, element):
     """
     Assign roles from role_id_list to element.
@@ -178,7 +185,6 @@ def assign_all_roles(acl_section, role_id_list, element):
         raise LibraryError(*report_list)
 
 
-
 def unassign_role(target_el, role_id, autodelete_target=False):
     """
     Unassign role with role_id from specified target/user target_el.
@@ -189,12 +195,19 @@ def unassign_role(target_el, role_id, autodelete_target=False):
     role_id -- id of role
     autodelete_target -- if True remove target_el if there is no role assigned
     """
-    assigned_role = target_el.find("./role[@id='{0}']".format(role_id))
-    if assigned_role is None:
-        raise LibraryError(reports.acl_role_is_not_assigned_to_target(
-            role_id, target_el.get("id")
-        ))
-    target_el.remove(assigned_role)
+    assigned_role_list = target_el.xpath(
+        "./role[@id=$role_id]", role_id=role_id
+    )
+    if not assigned_role_list:
+        raise LibraryError(
+            reports.ReportItem.error(
+                reports.messages.CibAclRoleIsNotAssignedToTarget(
+                    role_id, target_el.get("id")
+                )
+            )
+        )
+    for assigned_role in assigned_role_list:
+        target_el.remove(assigned_role)
     if autodelete_target and target_el.find("./role") is None:
         target_el.getparent().remove(target_el)
 
@@ -218,11 +231,14 @@ def create_target(acl_section, target_id):
     """
     # id of element acl_target is not type ID in CIB ACL schema so we don't need
     # to check if it is unique ID in whole CIB
-    if(
-        acl_section.find("./{0}[@id='{1}']".format(TAG_TARGET, target_id))
-        is not None
+    if acl_section.xpath(
+        f"./{TAG_TARGET}[@id=$target_id]", target_id=target_id
     ):
-        raise LibraryError(reports.acl_target_already_exists(target_id))
+        raise LibraryError(
+            reports.ReportItem.error(
+                reports.messages.CibAclTargetAlreadyExists(target_id)
+            )
+        )
     return etree.SubElement(acl_section, TAG_TARGET, id=target_id)
 
 
@@ -271,17 +287,16 @@ def add_permissions_to_role(role_el, permission_info_list):
         each contains (permission, scope_type, scope)
     """
     area_type_attribute_map = {
-        'xpath': 'xpath',
-        'id': 'reference',
+        "xpath": "xpath",
+        "id": "reference",
     }
     for permission, scope_type, scope in permission_info_list:
         perm = etree.SubElement(role_el, "acl_permission")
         perm.set(
             "id",
             find_unique_id(
-                role_el,
-                "{0}-{1}".format(role_el.get("id", "role"), permission)
-            )
+                role_el, "{0}-{1}".format(role_el.get("id", "role"), permission)
+            ),
         )
         perm.set("kind", permission)
         perm.set(area_type_attribute_map[scope_type], scope)
@@ -311,10 +326,8 @@ def get_role_list(acl_section):
     acl_section -- etree node
     """
     output_list = []
-    for role_el in acl_section.findall("./{0}".format(TAG_ROLE)):
-        role = etree_element_attibutes_to_dict(
-            role_el, ["id", "description"]
-        )
+    for role_el in acl_section.xpath(f"./{TAG_ROLE}"):
+        role = {key: role_el.get(key) for key in ["id", "description"]}
         role["permission_list"] = _get_permission_list(role_el)
         output_list.append(role)
     return output_list
@@ -323,7 +336,7 @@ def get_role_list(acl_section):
 def _get_permission_list(role_el):
     """
     Return list of all permissions of role element role_el.
-    Format of item of output list (if attribute is misssing in element under its
+    Format of item of output list (if attribute is missing in element under its
     key there is None):
         {
             "id": <id of permission element>,
@@ -335,17 +348,24 @@ def _get_permission_list(role_el):
             "attribute": <>,
         }
 
-    role_el -- acl_role etree element of which permissions whould be returned
+    role_el -- acl_role etree element of which permissions would be returned
     """
     output_list = []
     for permission in role_el.findall("./acl_permission"):
-        output_list.append(etree_element_attibutes_to_dict(
-            permission,
-            [
-                "id", "description", "kind", "xpath", "reference",
-                "object-type", "attribute"
-            ]
-        ))
+        output_list.append(
+            {
+                key: permission.get(key)
+                for key in [
+                    "id",
+                    "description",
+                    "kind",
+                    "xpath",
+                    "reference",
+                    "object-type",
+                    "attribute",
+                ]
+            }
+        )
     return output_list
 
 
@@ -377,11 +397,15 @@ def get_group_list(acl_section):
 
 def get_target_like_list(acl_section, tag):
     output_list = []
-    for target_el in acl_section.findall("./{0}".format(tag)):
-        output_list.append({
-            "id": target_el.get("id"),
-            "role_list": _get_role_list_of_target(target_el),
-        })
+    for target_el in acl_section.xpath(
+        "./*[local-name()=$tag_name]", tag_name=tag
+    ):
+        output_list.append(
+            {
+                "id": target_el.get("id"),
+                "role_list": _get_role_list_of_target(target_el),
+            }
+        )
     return output_list
 
 
@@ -404,8 +428,9 @@ def remove_permissions_referencing(tree, reference):
     tree -- etree node
     reference -- reference identifier
     """
-    xpath = './/acl_permission[@reference="{0}"]'.format(reference)
-    for permission in tree.findall(xpath):
+    for permission in tree.xpath(
+        ".//acl_permission[@reference=$reference]", reference=reference
+    ):
         permission.getparent().remove(permission)
 
 
