@@ -1,26 +1,37 @@
+from typing import (
+    List,
+    Optional,
+    cast,
+)
+
 from lxml import etree
+from lxml.etree import _Element
 
 from pcs.common import reports
-from pcs.common.reports.item import ReportItem
+from pcs.common.pacemaker.resource import bundle
 from pcs.lib import validate
+from pcs.lib.cib import (
+    nvpair_multi,
+    rule,
+)
+from pcs.lib.cib.const import TAG_RESOURCE_BUNDLE as TAG
 from pcs.lib.cib.nvpair import (
+    META_ATTRIBUTES_TAG,
     append_new_meta_attributes,
     arrange_first_meta_attributes,
-    META_ATTRIBUTES_TAG,
 )
 from pcs.lib.cib.resource.primitive import TAG as TAG_PRIMITIVE
 from pcs.lib.cib.tools import ElementSearcher
 from pcs.lib.errors import LibraryError
 from pcs.lib.pacemaker.values import sanitize_id
+from pcs.lib.tools import get_optional_value
 from pcs.lib.xml_tools import (
     append_when_useful,
     get_sub_element,
-    update_attributes_remove_empty,
     reset_element,
+    update_attributes_remove_empty,
 )
 
-TAG = "bundle"
-
 GENERIC_CONTAINER_TYPES = {"docker", "podman", "rkt"}
 
 GENERIC_CONTAINER_OPTIONS = frozenset(
@@ -65,10 +76,105 @@ STORAGE_MAP_OPTIONS = frozenset(
 )
 
 
-def is_bundle(resource_el):
+def is_bundle(resource_el: _Element) -> bool:
     return resource_el.tag == TAG
 
 
+def bundle_element_to_dto(
+    bundle_element: _Element,
+    rule_eval: Optional[rule.RuleInEffectEval] = None,
+) -> bundle.CibResourceBundleDto:
+    if rule_eval is None:
+        rule_eval = rule.RuleInEffectEvalDummy()
+    primitive_el = get_inner_resource(bundle_element)
+    runtime_el = _get_container_element(bundle_element)
+    network_el = bundle_element.find("network")
+    return bundle.CibResourceBundleDto(
+        id=str(bundle_element.attrib["id"]),
+        description=bundle_element.get("description"),
+        member_id=(
+            str(primitive_el.attrib["id"]) if primitive_el is not None else None
+        ),
+        container_type=(
+            bundle.ContainerType(runtime_el.tag)
+            if runtime_el is not None
+            else None
+        ),
+        container_options=(
+            bundle.CibResourceBundleContainerRuntimeOptionsDto(
+                image=str(runtime_el.attrib["image"]),
+                replicas=get_optional_value(int, runtime_el.get("replicas")),
+                replicas_per_host=get_optional_value(
+                    int, runtime_el.get("replicas-per-host")
+                ),
+                promoted_max=get_optional_value(
+                    int,
+                    runtime_el.get("promoted-max") or runtime_el.get("masters"),
+                ),
+                run_command=runtime_el.get("run-command"),
+                network=runtime_el.get("network"),
+                options=runtime_el.get("options"),
+            )
+            if runtime_el is not None
+            else None
+        ),
+        network=(
+            _bundle_network_element_to_dto(network_el)
+            if network_el is not None
+            else None
+        ),
+        port_mappings=[
+            bundle.CibResourceBundlePortMappingDto(
+                id=str(net_map_el.attrib["id"]),
+                port=get_optional_value(int, net_map_el.get("port")),
+                internal_port=get_optional_value(
+                    int, net_map_el.get("internal-port")
+                ),
+                range=net_map_el.get("range"),
+            )
+            for net_map_el in bundle_element.findall("network/port-mapping")
+        ],
+        storage_mappings=[
+            bundle.CibResourceBundleStorageMappingDto(
+                id=str(storage_el.attrib["id"]),
+                source_dir=storage_el.get("source-dir"),
+                source_dir_root=storage_el.get("source-dir-root"),
+                target_dir=str(storage_el.attrib["target-dir"]),
+                options=storage_el.get("options"),
+            )
+            for storage_el in bundle_element.findall("storage/storage-mapping")
+        ],
+        meta_attributes=[
+            nvpair_multi.nvset_element_to_dto(nvset, rule_eval)
+            for nvset in nvpair_multi.find_nvsets(
+                bundle_element, nvpair_multi.NVSET_META
+            )
+        ],
+        instance_attributes=[
+            nvpair_multi.nvset_element_to_dto(nvset, rule_eval)
+            for nvset in nvpair_multi.find_nvsets(
+                bundle_element, nvpair_multi.NVSET_INSTANCE
+            )
+        ],
+    )
+
+
+def _bundle_network_element_to_dto(
+    network_element: _Element,
+) -> bundle.CibResourceBundleNetworkOptionsDto:
+    return bundle.CibResourceBundleNetworkOptionsDto(
+        ip_range_start=network_element.get("ip-range-start"),
+        control_port=get_optional_value(
+            int, network_element.get("control-port")
+        ),
+        host_interface=network_element.get("host-interface"),
+        host_netmask=get_optional_value(
+            int, network_element.get("host-netmask")
+        ),
+        add_host=get_optional_value(bool, network_element.get("add-host")),
+    )
+
+
 def validate_new(
     id_provider,
     bundle_id,
@@ -213,7 +319,7 @@ def reset_to_minimal(bundle_element):
 
 
 def _get_report_unsupported_container(bundle_el):
-    return ReportItem.error(
+    return reports.ReportItem.error(
         reports.messages.ResourceBundleUnsupportedContainerType(
             bundle_el.get("id"),
             sorted(GENERIC_CONTAINER_TYPES),
@@ -372,7 +478,7 @@ def add_resource(bundle_element, primitive_element):
     inner_primitive = bundle_element.find(TAG_PRIMITIVE)
     if inner_primitive is not None:
         raise LibraryError(
-            ReportItem.error(
+            reports.ReportItem.error(
                 reports.messages.ResourceBundleAlreadyContainsAResource(
                     bundle_element.get("id"),
                     inner_primitive.get("id"),
@@ -382,8 +488,8 @@ def add_resource(bundle_element, primitive_element):
     bundle_element.append(primitive_element)
 
 
-def get_inner_resource(bundle_el):
-    resources = bundle_el.xpath("./primitive")
+def get_inner_resource(bundle_el: _Element) -> Optional[_Element]:
+    resources = cast(List[_Element], bundle_el.xpath("./primitive"))
     if resources:
         return resources[0]
     return None
@@ -398,7 +504,7 @@ def _is_supported_container(container_el):
 def _validate_container(container_type, container_options, force_options=False):
     if container_type not in GENERIC_CONTAINER_TYPES:
         return [
-            ReportItem.error(
+            reports.ReportItem.error(
                 reports.messages.InvalidOptionValue(
                     "container type",
                     container_type,
@@ -433,7 +539,7 @@ def _validate_generic_container_options(container_options, force_options=False):
     deprecation_reports = []
     if "masters" in container_options:
         deprecation_reports.append(
-            ReportItem.warning(
+            reports.ReportItem.deprecation(
                 reports.messages.DeprecatedOption(
                     "masters",
                     ["promoted-max"],
@@ -516,7 +622,7 @@ def _validate_generic_container_options_update(
         # deprecated. They may be removing it because they just found out it is
         # deprecated.
         deprecation_reports.append(
-            ReportItem.warning(
+            reports.ReportItem.deprecation(
                 reports.messages.DeprecatedOption(
                     "masters",
                     ["promoted-max"],
@@ -533,7 +639,7 @@ def _validate_generic_container_options_update(
         and options.get("promoted-max") != ""
     ):
         deprecation_reports.append(
-            ReportItem.error(
+            reports.ReportItem.error(
                 reports.messages.PrerequisiteOptionMustNotBeSet(
                     "masters", "promoted-max", "container", "container"
                 )
@@ -545,7 +651,7 @@ def _validate_generic_container_options_update(
         and options.get("masters") != ""
     ):
         deprecation_reports.append(
-            ReportItem.error(
+            reports.ReportItem.error(
                 reports.messages.PrerequisiteOptionMustNotBeSet(
                     "promoted-max", "masters", "container", "container"
                 )
@@ -573,7 +679,7 @@ def _validate_network_options_new(options, force_options):
     return validate.ValidatorAll(validators).validate(options)
 
 
-def _is_pcmk_remote_acccessible_after_update(network_el, options):
+def _is_pcmk_remote_accessible_after_update(network_el, options):
     port_name = "control-port"
     ip_name = "ip-range-start"
     port = network_el.get(port_name)
@@ -608,10 +714,10 @@ def _validate_network_options_update(
     inner_primitive = get_inner_resource(bundle_el)
     if (
         inner_primitive is not None
-        and not _is_pcmk_remote_acccessible_after_update(network_el, options)
+        and not _is_pcmk_remote_accessible_after_update(network_el, options)
     ):
         report_list.append(
-            ReportItem(
+            reports.ReportItem(
                 severity=reports.item.get_severity(
                     reports.codes.FORCE,
                     force_options,
