@@ -1,15 +1,27 @@
 import re
 from typing import (
-    cast,
+    Iterable,
     List,
+    Pattern,
     Set,
+    Tuple,
+    cast,
 )
 
-from lxml.etree import _Element
+from lxml.etree import (
+    _Element,
+    _ElementTree,
+)
 
-from pcs.common import reports
+from pcs.common import (
+    const,
+    reports,
+)
 from pcs.common.reports import codes as report_codes
-from pcs.common.reports.item import ReportItem, ReportItemList
+from pcs.common.reports.item import (
+    ReportItem,
+    ReportItemList,
+)
 from pcs.common.tools import Version
 from pcs.lib.cib import sections
 from pcs.lib.errors import LibraryError
@@ -17,9 +29,16 @@ from pcs.lib.pacemaker.values import (
     sanitize_id,
     validate_id,
 )
-from pcs.lib.xml_tools import get_root, get_sub_element
+from pcs.lib.xml_tools import (
+    get_root,
+    get_sub_element,
+)
 
-VERSION_FORMAT = r"(?P<major>\d+)\.(?P<minor>\d+)(\.(?P<rev>\d+))?$"
+_VERSION_FORMAT = r"(?P<major>\d+)\.(?P<minor>\d+)(\.(?P<rev>\d+))?$"
+
+
+class ElementNotFound(Exception):
+    pass
 
 
 class IdProvider:
@@ -63,6 +82,7 @@ class IdProvider:
         return report_list
 
 
+# DEPRECATED, use get_element(s)_by_id(s) instead
 class ElementSearcher:
     """
     Search for an element, allow to book its id if not found, provide reports
@@ -248,6 +268,45 @@ def get_configuration_elements_by_id(
     )
 
 
+def get_element_by_id(cib: _Element, element_id: str) -> _Element:
+    """
+    Returns an element from CIB with the given IDs
+
+    cib -- the whole cib
+    element_id -- element ID to look for
+    """
+    element_list = get_configuration_elements_by_id(cib, element_id)
+    if not element_list:
+        raise ElementNotFound
+    if len(element_list) > 1:
+        # List should only contain one element - IDs need to be unique across
+        # the whole CIB since Pacemaker does XML schema validation
+        raise AssertionError(
+            f"Found more than one match for id '{element_id}' in the CIB"
+        )
+    return element_list[0]
+
+
+def get_elements_by_ids(
+    cib: _Element, element_ids: Iterable[str]
+) -> Tuple[List[_Element], List[str]]:
+    """
+    Returns a list of elements from CIB with the given IDs and a list of IDs
+    that weren't found
+
+    cib -- the whole cib
+    element_ids -- iterable with element IDs to look for
+    """
+    found_element_list = []
+    id_not_found_list = []
+    for element_id in element_ids:
+        try:
+            found_element_list.append(get_element_by_id(cib, element_id))
+        except ElementNotFound:
+            id_not_found_list.append(element_id)
+    return found_element_list, id_not_found_list
+
+
 # DEPRECATED, use IdProvider instead
 def does_id_exist(tree, check_id):
     """
@@ -420,11 +479,11 @@ def get_tags(tree: _Element) -> _Element:
     return sections.get(tree, sections.TAGS)
 
 
-def _get_cib_version(cib, attribute, regexp, none_if_missing=False):
-    version = cib.get(attribute)
+def _get_cib_version(
+    cib: _ElementTree, attribute: str, regexp: Pattern
+) -> Version:
+    version = cib.getroot().get(attribute)
     if version is None:
-        if none_if_missing:
-            return None
         raise LibraryError(
             ReportItem.error(
                 reports.messages.CibLoadErrorBadFormat(
@@ -450,7 +509,7 @@ def _get_cib_version(cib, attribute, regexp, none_if_missing=False):
     )
 
 
-def get_pacemaker_version_by_which_cib_was_validated(cib):
+def get_pacemaker_version_by_which_cib_was_validated(cib: _Element) -> Version:
     """
     Return version of pacemaker which validated specified cib as tree.
     Version is returned as an instance of pcs.common.tools.Version.
@@ -459,23 +518,25 @@ def get_pacemaker_version_by_which_cib_was_validated(cib):
     cib -- cib etree
     """
     return _get_cib_version(
-        cib,
+        cib.getroottree(),
         "validate-with",
-        re.compile(r"pacemaker-{0}".format(VERSION_FORMAT)),
+        re.compile(r"pacemaker-{0}".format(_VERSION_FORMAT)),
     )
 
 
-def get_cib_crm_feature_set(cib, none_if_missing=False):
+def get_cib_crm_feature_set(cib: _Element) -> Version:
     """
     Return crm_feature_set as pcs.common.tools.Version or raise LibraryError
 
     etree cib -- cib etree
-    bool none_if_missing -- return None instead of raising when crm_feature_set
-        is missing
     """
     return _get_cib_version(
-        cib,
-        "crm_feature_set",
-        re.compile(VERSION_FORMAT),
-        none_if_missing=none_if_missing,
+        cib.getroottree(), "crm_feature_set", re.compile(_VERSION_FORMAT)
+    )
+
+
+def are_new_role_names_supported(cib: _Element) -> bool:
+    return (
+        get_pacemaker_version_by_which_cib_was_validated(cib)
+        >= const.PCMK_NEW_ROLES_CIB_VERSION
     )
