@@ -1,5 +1,6 @@
 from typing import (
     Any,
+    Collection,
     Container,
     Iterable,
     List,
@@ -7,7 +8,10 @@ from typing import (
     Optional,
 )
 
+from lxml.etree import _Element
+
 from pcs.common import reports
+from pcs.common.pacemaker.defaults import CibDefaultsDto
 from pcs.common.pacemaker.nvset import CibNvsetDto
 from pcs.common.reports.item import ReportItem
 from pcs.common.tools import Version
@@ -30,6 +34,7 @@ from pcs.lib.cib.tools import (
 )
 from pcs.lib.env import LibraryEnvironment
 from pcs.lib.errors import LibraryError
+from pcs.lib.external import CommandRunner
 from pcs.lib.pacemaker.live import has_rule_in_effect_status_tool
 
 
@@ -38,7 +43,7 @@ def resource_defaults_create(
     nvpairs: Mapping[str, str],
     nvset_options: Mapping[str, str],
     nvset_rule: Optional[str] = None,
-    force_flags: Container[reports.types.ForceCode] = (),
+    force_flags: Collection[reports.types.ForceCode] = (),
 ) -> None:
     """
     Create new resource defaults nvset
@@ -69,7 +74,7 @@ def operation_defaults_create(
     nvpairs: Mapping[str, str],
     nvset_options: Mapping[str, str],
     nvset_rule: Optional[str] = None,
-    force_flags: Container[reports.types.ForceCode] = (),
+    force_flags: Collection[reports.types.ForceCode] = (),
 ) -> None:
     """
     Create new operation defaults nvset
@@ -148,66 +153,94 @@ def _defaults_create(
     )
 
     env.report_processor.report(
-        ReportItem.warning(reports.messages.DefaultsCanBeOverriden())
+        ReportItem.warning(reports.messages.DefaultsCanBeOverridden())
     )
     env.push_cib()
 
 
 def resource_defaults_config(
     env: LibraryEnvironment, evaluate_expired: bool
-) -> List[CibNvsetDto]:
+) -> CibDefaultsDto:
     """
     List all resource defaults nvsets
 
     env --
     evaluate_expired -- also evaluate whether rules are expired or in effect
     """
-    return _defaults_config(env, sections.RSC_DEFAULTS, evaluate_expired)
+    cib = env.get_cib()
+    rule_evaluator = _get_rule_evaluator(
+        cib, env.cmd_runner(), env.report_processor, evaluate_expired
+    )
+    get_config = lambda tag: _defaults_config(
+        cib,
+        tag,
+        sections.RSC_DEFAULTS,
+        rule_evaluator,
+    )
+    return CibDefaultsDto(
+        instance_attributes=get_config(nvpair_multi.NVSET_INSTANCE),
+        meta_attributes=get_config(nvpair_multi.NVSET_META),
+    )
 
 
 def operation_defaults_config(
     env: LibraryEnvironment, evaluate_expired: bool
-) -> List[CibNvsetDto]:
+) -> CibDefaultsDto:
     """
     List all operation defaults nvsets
 
     env --
     evaluate_expired -- also evaluate whether rules are expired or in effect
     """
-    return _defaults_config(env, sections.OP_DEFAULTS, evaluate_expired)
+    cib = env.get_cib()
+    rule_evaluator = _get_rule_evaluator(
+        cib, env.cmd_runner(), env.report_processor, evaluate_expired
+    )
+    get_config = lambda tag: _defaults_config(
+        cib,
+        tag,
+        sections.OP_DEFAULTS,
+        rule_evaluator,
+    )
+    return CibDefaultsDto(
+        instance_attributes=get_config(nvpair_multi.NVSET_INSTANCE),
+        meta_attributes=get_config(nvpair_multi.NVSET_META),
+    )
+
+
+def _get_rule_evaluator(
+    cib: _Element,
+    runner: CommandRunner,
+    report_processor: reports.ReportProcessor,
+    evaluate_expired: bool,
+) -> RuleInEffectEval:
+    if evaluate_expired:
+        if has_rule_in_effect_status_tool():
+            return RuleInEffectEvalOneByOne(cib, runner)
+        report_processor.report(
+            ReportItem.warning(
+                reports.messages.RuleInEffectStatusDetectionNotSupported()
+            )
+        )
+    return RuleInEffectEvalDummy()
 
 
 def _defaults_config(
-    env: LibraryEnvironment, cib_section_name: str, evaluate_expired: bool
+    cib: _Element,
+    nvset_tag: nvpair_multi.NvsetTag,
+    cib_section_name: str,
+    rule_evaluator: RuleInEffectEval,
 ) -> List[CibNvsetDto]:
-    runner = env.cmd_runner()
-    cib = env.get_cib()
-
-    if evaluate_expired:
-        if has_rule_in_effect_status_tool():
-            in_effect_eval: RuleInEffectEval = RuleInEffectEvalOneByOne(
-                cib, runner
-            )
-        else:
-            in_effect_eval = RuleInEffectEvalDummy()
-            env.report_processor.report(
-                ReportItem.warning(
-                    reports.messages.RuleInEffectStatusDetectionNotSupported()
-                )
-            )
-    else:
-        in_effect_eval = RuleInEffectEvalDummy()
-
     return [
-        nvpair_multi.nvset_element_to_dto(nvset_el, in_effect_eval)
+        nvpair_multi.nvset_element_to_dto(nvset_el, rule_evaluator)
         for nvset_el in nvpair_multi.find_nvsets(
-            sections.get(cib, cib_section_name)
+            sections.get(cib, cib_section_name), nvset_tag
         )
     ]
 
 
 def resource_defaults_remove(
-    env: LibraryEnvironment, nvset_id_list: Iterable[str]
+    env: LibraryEnvironment, nvset_id_list: Collection[str]
 ) -> None:
     """
     Remove specified resource defaults nvsets
@@ -219,7 +252,7 @@ def resource_defaults_remove(
 
 
 def operation_defaults_remove(
-    env: LibraryEnvironment, nvset_id_list: Iterable[str]
+    env: LibraryEnvironment, nvset_id_list: Collection[str]
 ) -> None:
     """
     Remove specified operation defaults nvsets
@@ -322,12 +355,12 @@ def _defaults_update(
                 break
         if only_removing and not sections.exists(cib, cib_section_name):
             env.report_processor.report(
-                ReportItem.warning(reports.messages.DefaultsCanBeOverriden())
+                ReportItem.warning(reports.messages.DefaultsCanBeOverridden())
             )
             return
 
         nvset_elements = nvpair_multi.find_nvsets(
-            sections.get(cib, cib_section_name)
+            sections.get(cib, cib_section_name), nvpair_multi.NVSET_META
         )
         if len(nvset_elements) > 1:
             env.report_processor.report(
@@ -339,7 +372,7 @@ def _defaults_update(
             )
             raise LibraryError()
         env.report_processor.report(
-            ReportItem.warning(reports.messages.DefaultsCanBeOverriden())
+            ReportItem.warning(reports.messages.DefaultsCanBeOverridden())
         )
         if len(nvset_elements) == 1:
             nvpair_multi.nvset_update(nvset_elements[0], id_provider, nvpairs)
@@ -367,6 +400,6 @@ def _defaults_update(
 
     nvpair_multi.nvset_update(nvset_elements[0], id_provider, nvpairs)
     env.report_processor.report(
-        ReportItem.warning(reports.messages.DefaultsCanBeOverriden())
+        ReportItem.warning(reports.messages.DefaultsCanBeOverridden())
     )
     env.push_cib()
