@@ -1,36 +1,62 @@
 from unittest import TestCase
+
 from lxml import etree
 
-from pcs_test.tools.cib import get_assert_pcs_effect_mixin
+from pcs_test.tools.cib import (
+    get_assert_pcs_effect_mixin_old as get_assert_pcs_effect_mixin,
+)
+from pcs_test.tools.misc import get_test_resource as rc
 from pcs_test.tools.misc import (
-    get_test_resource as rc,
     get_tmp_file,
     write_data_to_tmpfile,
 )
-from pcs_test.tools.pcs_runner import PcsRunner
+from pcs_test.tools.pcs_runner import PcsRunnerOld as PcsRunner
 from pcs_test.tools.xml import XmlManipulation
 
 
-FIXTURE_DUMMY = """
-    <primitive class="ocf" id="Dummy" provider="heartbeat" type="Dummy">
-        <operations>
-            <op id="Dummy-monitor-interval-10s" interval="10s"
-                name="monitor" timeout="20s"/>
-        </operations>
-    </primitive>
-"""
+def _get_primitive_fixture(
+    res_id, agent_standard="ocf", agent_provider="heartbeat", agent_type="Dummy"
+):
+    _provider = ""
+    if agent_provider:
+        _provider = f' provider="{agent_provider}"'
+    return f"""
+        <primitive class="{agent_standard}" id="{res_id}"{_provider} type="{agent_type}">
+            <operations>
+                <op id="{res_id}-monitor-interval-10s" interval="10s"
+                    name="monitor" timeout="20s"/>
+            </operations>
+        </primitive>
+    """
 
-FIXTURE_PRIMITIVE_FOR_CLONE = """
-    <primitive class="ocf" id="C" provider="heartbeat" type="Dummy">
-        <operations>
-            <op id="C-monitor-interval-10s" interval="10s"
-                name="monitor" timeout="20s"/>
-        </operations>
-    </primitive>
-"""
+
+FIXTURE_DUMMY = _get_primitive_fixture("Dummy")
+FIXTURE_PRIMITIVE_FOR_CLONE = _get_primitive_fixture("C")
 
 FIXTURE_CLONE = f"""<clone id="C-clone">{FIXTURE_PRIMITIVE_FOR_CLONE}</clone>"""
 
+FIXTURE_STONITH_FOR_CLONE = """
+    <primitive class="stonith" id="fence-device" type="fence_xvm">
+        <operations>
+          <op id="fence-device-monitor-interval-60s" interval="60s"
+              name="monitor"/>
+        </operations>
+    </primitive>
+"""
+
+FIXTURE_STONITH_CLONE = f"""
+    <clone id="fence-device-clone">{FIXTURE_STONITH_FOR_CLONE}</clone>
+"""
+
+FIXTURE_STONITH_PROMOTABLE = f"""
+    <clone id="fence-device-clone">
+        {FIXTURE_STONITH_FOR_CLONE}
+        <meta_attributes id="fence-device-clone-meta_attributes">
+            <nvpair id="fence-device-clone-meta_attributes-promotable"
+                name="promotable" value="true"></nvpair>
+        </meta_attributes>
+    </clone>
+"""
 FIXTURE_CLONE_WITH_OPTIONS = f"""
     <clone id="CustomCloneId">
         {FIXTURE_PRIMITIVE_FOR_CLONE}
@@ -62,13 +88,17 @@ FIXTURE_CLONED_GROUP = """
     </clone>
 """
 
-FIXTURE_GROUP_LAST_MEMBER = """
-    <group id="Group">
-        {}
-    </group>
-""".format(
-    FIXTURE_PRIMITIVE_FOR_CLONE
-)
+FIXTURE_GROUP_WITH_STONITH = f"""
+    <group id="Group">{FIXTURE_STONITH_FOR_CLONE}</group>
+"""
+
+FIXTURE_CLONED_GROUP_WITH_STONITH = f"""
+    <clone id="Group-clone">{FIXTURE_GROUP_WITH_STONITH}</clone>
+"""
+
+FIXTURE_GROUP_LAST_MEMBER = f"""
+    <group id="Group">{FIXTURE_PRIMITIVE_FOR_CLONE}</group>
+"""
 
 FIXTURE_CONSTRAINTS_CONFIG_XML = """
     <constraints>
@@ -157,6 +187,18 @@ FIXTURE_RESOURCES = fixture_resources_xml(
 )
 
 
+def fixture_clone_stonith_msg(forced=False, group=False):
+    return (
+        "{severity}: {group}No need to clone stonith resource 'fence-device', any node"
+        " can use a stonith resource (unless specifically banned) regardless of "
+        "whether the stonith resource is running on that node or not{use_force}"
+    ).format(
+        severity="Warning" if forced else "Error",
+        group="Group 'Group' contains stonith resource. " if group else "",
+        use_force="\n" if forced else ", use " "--force to override\n",
+    )
+
+
 class Unclone(
     TestCase,
     get_assert_pcs_effect_mixin(
@@ -240,7 +282,7 @@ class Unclone(
         self.assert_tags_xml(FIXTURE_TAGS_RESULT_XML)
         self.assert_constraint_xml("<constraints/>")
 
-    def test_unclone_resoruce_id(self):
+    def test_unclone_resource_id(self):
         self.assert_effect("resource unclone C".split(), FIXTURE_RESOURCES)
         self.assert_tags_xml(FIXTURE_TAGS_RESULT_XML)
         self.assert_constraint_xml("<constraints/>")
@@ -255,12 +297,17 @@ class Clone(
         )
     ),
 ):
+    # pylint: disable=too-many-public-methods
     empty_cib = rc("cib-empty.xml")
 
     def setUp(self):
         self.temp_cib = get_tmp_file("tier1_cib_resource_clone_unclone_clone")
         self.pcs_runner = PcsRunner(self.temp_cib.name)
         self.set_cib_file(FIXTURE_PRIMITIVE_FOR_CLONE)
+        self.stonith_deprecation_warning = (
+            "Deprecation Warning: Ability of this command to accept stonith "
+            "resources is deprecated and will be removed in a future release.\n"
+        )
 
     def tearDown(self):
         self.temp_cib.close()
@@ -295,6 +342,108 @@ class Clone(
             ),
         )
 
+    def test_clone_id_is_stonith(self):
+        self.set_cib_file(FIXTURE_STONITH_FOR_CLONE)
+        self.assert_pcs_fail(
+            "resource clone fence-device".split(),
+            self.stonith_deprecation_warning + fixture_clone_stonith_msg(),
+        )
+        self.assert_resources_xml_in_cib(
+            fixture_resources_xml(FIXTURE_STONITH_FOR_CLONE)
+        )
+
+    def test_clone_id_is_stonith_forced(self):
+        self.set_cib_file(FIXTURE_STONITH_FOR_CLONE)
+        self.assert_effect(
+            "resource clone fence-device --force".split(),
+            fixture_resources_xml(FIXTURE_STONITH_CLONE),
+            output=self.stonith_deprecation_warning
+            + fixture_clone_stonith_msg(forced=True),
+        )
+
+    def test_clone_group_with_stonith(self):
+        self.set_cib_file(FIXTURE_GROUP_WITH_STONITH)
+        self.assert_effect(
+            "resource clone Group".split(),
+            fixture_resources_xml(FIXTURE_CLONED_GROUP_WITH_STONITH),
+        )
+
+    def test_clone_group_with_stonith_forced(self):
+        self.set_cib_file(FIXTURE_GROUP_WITH_STONITH)
+        self.assert_effect(
+            "resource clone Group --force".split(),
+            fixture_resources_xml(FIXTURE_CLONED_GROUP_WITH_STONITH),
+        )
+
+    def test_clone_globally_unique_not_ocf_agent(self):
+        self.set_cib_file(
+            _get_primitive_fixture(
+                "C",
+                agent_standard="systemd",
+                agent_provider=None,
+                agent_type="pacemaker",
+            )
+        )
+        self.assert_pcs_fail(
+            "resource clone C globally-unique=true".split(),
+            stdout_full=(
+                "Error: Clone option 'globally-unique' is not compatible with "
+                "'systemd:pacemaker' resource agent of resource 'C'\n"
+            ),
+        )
+
+    def test_clone_promotable_group_some_unsupported(self):
+        self.set_cib_file(
+            '<group id="G">'
+            + "\n".join(
+                [
+                    _get_primitive_fixture(
+                        "A",
+                        agent_standard="systemd",
+                        agent_provider=None,
+                        agent_type="pacemaker",
+                    ),
+                    _get_primitive_fixture(
+                        "B",
+                        agent_provider="pacemaker",
+                        agent_type="Stateful",
+                    ),
+                    _get_primitive_fixture(
+                        "C",
+                        agent_provider="pacemaker",
+                    ),
+                ]
+            )
+            + "</group>"
+        )
+        self.assert_pcs_fail(
+            "resource clone G promotable=true".split(),
+            stdout_full=(
+                "Error: Clone option 'promotable' is not compatible with "
+                "'systemd:pacemaker' resource agent of resource 'A' in group "
+                "'G'\nError: Clone option 'promotable' is not compatible with "
+                "'ocf:pacemaker:Dummy' resource agent of resource 'C' in group "
+                "'G', use --force to override\n"
+            ),
+        )
+
+    def test_clone_promotable_not_ocf_agent(self):
+        self.set_cib_file(
+            _get_primitive_fixture(
+                "C",
+                agent_standard="systemd",
+                agent_provider=None,
+                agent_type="pacemaker",
+            )
+        )
+        self.assert_pcs_fail(
+            "resource clone C promotable=true".split(),
+            stdout_full=(
+                "Error: Clone option 'promotable' is not compatible with "
+                "'systemd:pacemaker' resource agent of resource 'C'\n"
+            ),
+        )
+
     def test_promotable_clone(self):
         self.assert_effect(
             "resource promotable C".split(),
@@ -303,6 +452,36 @@ class Clone(
             ),
         )
 
+    def test_promotable_clone_not_ocf_agent(self):
+        self.set_cib_file(
+            _get_primitive_fixture(
+                "C",
+                agent_standard="systemd",
+                agent_provider=None,
+                agent_type="pacemaker",
+            )
+        )
+        self.assert_pcs_fail(
+            "resource promotable C".split(),
+            stdout_full=(
+                "Error: Clone option 'promotable' is not compatible with "
+                "'systemd:pacemaker' resource agent of resource 'C'\n"
+            ),
+        )
+
+    def test_promotable_clone_unsupported_agent(self):
+        self.set_cib_file(
+            _get_primitive_fixture("C", agent_provider="pacemaker")
+        )
+        self.assert_pcs_fail(
+            "resource promotable C".split(),
+            stdout_full=(
+                "Error: Clone option 'promotable' is not compatible with "
+                "'ocf:pacemaker:Dummy' resource agent of resource 'C', use "
+                "--force to override\n"
+            ),
+        )
+
     def test_promotable_clone_custom_id(self):
         self.assert_effect(
             "resource promotable C CustomPromotableId".split(),
@@ -311,6 +490,24 @@ class Clone(
             ),
         )
 
+    def test_promotable_clone_id_is_stonith(self):
+        self.set_cib_file(FIXTURE_STONITH_FOR_CLONE)
+        self.assert_pcs_fail(
+            "resource promotable fence-device".split(),
+            self.stonith_deprecation_warning + fixture_clone_stonith_msg(),
+        )
+        self.assert_resources_xml_in_cib(
+            fixture_resources_xml(FIXTURE_STONITH_FOR_CLONE)
+        )
+
+    def test_promotable_clone_id_is_stonith_forced(self):
+        self.set_cib_file(FIXTURE_STONITH_FOR_CLONE)
+        self.assert_pcs_fail(
+            "resource promotable fence-device --force".split(),
+            stdout_start=self.stonith_deprecation_warning
+            + fixture_clone_stonith_msg(forced=True),
+        )
+
     def test_promotable_keyword_and_option(self):
         self.assert_pcs_fail(
             "resource promotable C CustomCloneId promotable=false".split(),
