@@ -1,22 +1,31 @@
 # pylint: disable=too-many-lines
-from unittest import TestCase
 import xml.dom.minidom
+from unittest import TestCase
 
-from pcs import rule, utils
-from pcs.common.tools import Version
-from pcs_test.tools.assertions import ac, assert_xml_equal
+from pcs import (
+    rule,
+    utils,
+)
+from pcs.common import const
+from pcs.common.str_tools import format_list_custom_last_separator
+
+from pcs_test.tools.assertions import (
+    ac,
+    assert_xml_equal,
+)
+from pcs_test.tools.misc import get_test_resource as rc
 from pcs_test.tools.misc import (
-    get_test_resource as rc,
     get_tmp_file,
+    skip_unless_crm_rule,
     write_file_to_tmpfile,
 )
-from pcs_test.tools.misc import skip_unless_crm_rule
-from pcs_test.tools.pcs_runner import pcs
+from pcs_test.tools.pcs_runner import pcs_old as pcs
 
 # pylint: disable=invalid-name
 # pylint: disable=line-too-long
 
 empty_cib = rc("cib-empty-3.2.xml")
+empty_cib_new_roles_supported = rc("cib-empty-3.7.xml")
 
 
 class DateValueTest(TestCase):
@@ -1665,7 +1674,7 @@ class CibBuilderTest(TestCase):
         )
         constraint_el.setAttribute("id", "location-dummy")
         assert_xml_equal(
-            rule.CibBuilder(Version(*utils.getValidateWithVersion(cib_dom)))
+            rule.CibBuilder(utils.getValidateWithVersion(cib_dom))
             .build(constraint_el, rule.RuleParser().parse(rule_expression))
             .parentNode.toprettyxml(indent="    "),
             rule_xml.lstrip().rstrip(" "),
@@ -1678,6 +1687,7 @@ class TokenPreprocessorTest(TestCase):
 
     def testNoChanges(self):
         self.assertEqual([], self.preprocessor.run([]))
+
         self.assertEqual(
             ["#uname", "eq", "node1"],
             self.preprocessor.run(["#uname", "eq", "node1"]),
@@ -1685,24 +1695,29 @@ class TokenPreprocessorTest(TestCase):
 
     def testDateSpec(self):
         self.assertEqual(["date-spec"], self.preprocessor.run(["date-spec"]))
+
         self.assertEqual(
             ["date-spec", "hours=14"],
             self.preprocessor.run(["date-spec", "hours=14"]),
         )
+
         self.assertEqual(
             ["date-spec", "hours weeks=6 months= moon=1"],
             self.preprocessor.run(
                 ["date-spec", "hours", "weeks=6", "months=", "moon=1"]
             ),
         )
+
         self.assertEqual(
             ["date-spec", "foo", "hours=14"],
             self.preprocessor.run(["date-spec", "foo", "hours=14"]),
         )
+
         self.assertEqual(
             ["date-spec", "hours=14", "foo", "hours=14"],
             self.preprocessor.run(["date-spec", "hours=14", "foo", "hours=14"]),
         )
+
         self.assertEqual(
             [
                 "date-spec",
@@ -1724,12 +1739,14 @@ class TokenPreprocessorTest(TestCase):
                 ]
             ),
         )
+
         self.assertEqual(
             ["#uname", "eq", "node1", "or", "date-spec", "hours=14"],
             self.preprocessor.run(
                 ["#uname", "eq", "node1", "or", "date-spec", "hours=14"]
             ),
         )
+
         self.assertEqual(
             ["date-spec", "hours=14", "or", "#uname", "eq", "node1"],
             self.preprocessor.run(
@@ -1746,24 +1763,29 @@ class TokenPreprocessorTest(TestCase):
 
     def testDuration(self):
         self.assertEqual(["duration"], self.preprocessor.run(["duration"]))
+
         self.assertEqual(
             ["duration", "hours=14"],
             self.preprocessor.run(["duration", "hours=14"]),
         )
+
         self.assertEqual(
             ["duration", "hours weeks=6 months= moon=1"],
             self.preprocessor.run(
                 ["duration", "hours", "weeks=6", "months=", "moon=1"]
             ),
         )
+
         self.assertEqual(
             ["duration", "foo", "hours=14"],
             self.preprocessor.run(["duration", "foo", "hours=14"]),
         )
+
         self.assertEqual(
             ["duration", "hours=14", "foo", "hours=14"],
             self.preprocessor.run(["duration", "hours=14", "foo", "hours=14"]),
         )
+
         self.assertEqual(
             [
                 "duration",
@@ -1785,12 +1807,14 @@ class TokenPreprocessorTest(TestCase):
                 ]
             ),
         )
+
         self.assertEqual(
             ["#uname", "eq", "node1", "or", "duration", "hours=14"],
             self.preprocessor.run(
                 ["#uname", "eq", "node1", "or", "duration", "hours=14"]
             ),
         )
+
         self.assertEqual(
             ["duration", "hours=14", "or", "#uname", "eq", "node1"],
             self.preprocessor.run(
@@ -1805,204 +1829,58 @@ class TokenPreprocessorTest(TestCase):
             ),
         )
 
-    def testOperationDatespec(self):
-        self.assertEqual(
-            ["date-spec", "weeks=6 moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "operation=date_spec", "weeks=6", "moon=1"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6 moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "operation=date_spec", "moon=1"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6", "foo", "moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "operation=date_spec", "foo", "moon=1"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6", "foo", "operation=date_spec", "moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "foo", "operation=date_spec", "moon=1"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6 moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "moon=1", "operation=date_spec"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6 moon=1", "foo"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "moon=1", "operation=date_spec", "foo"]
-            ),
-        )
-        self.assertEqual(
-            ["date-spec"],
-            self.preprocessor.run(["date-spec", "operation=date_spec"]),
-        )
-        self.assertEqual(
-            ["date-spec", "weeks=6", "operation=foo", "moon=1"],
-            self.preprocessor.run(
-                ["date-spec", "weeks=6", "operation=foo", "moon=1"]
-            ),
-        )
-
-    def testDateLegacySyntax(self):
-        # valid syntax
-        self.assertEqual(
-            ["date", "gt", "2014-06-26"],
-            self.preprocessor.run(["date", "start=2014-06-26", "gt"]),
-        )
-        self.assertEqual(
-            ["date", "lt", "2014-06-26"],
-            self.preprocessor.run(["date", "end=2014-06-26", "lt"]),
-        )
-        self.assertEqual(
-            ["date", "in_range", "2014-06-26", "to", "2014-07-26"],
-            self.preprocessor.run(
-                ["date", "start=2014-06-26", "end=2014-07-26", "in_range"]
-            ),
-        )
-        self.assertEqual(
-            ["date", "in_range", "2014-06-26", "to", "2014-07-26"],
-            self.preprocessor.run(
-                ["date", "end=2014-07-26", "start=2014-06-26", "in_range"]
-            ),
-        )
-
-        self.assertEqual(
-            ["date", "gt", "2014-06-26", "foo"],
-            self.preprocessor.run(["date", "start=2014-06-26", "gt", "foo"]),
-        )
-        self.assertEqual(
-            ["date", "lt", "2014-06-26", "foo"],
-            self.preprocessor.run(["date", "end=2014-06-26", "lt", "foo"]),
-        )
-        self.assertEqual(
-            ["date", "in_range", "2014-06-26", "to", "2014-07-26", "foo"],
-            self.preprocessor.run(
-                [
-                    "date",
-                    "start=2014-06-26",
-                    "end=2014-07-26",
-                    "in_range",
-                    "foo",
-                ]
-            ),
-        )
-        self.assertEqual(
-            ["date", "in_range", "2014-06-26", "to", "2014-07-26", "foo"],
-            self.preprocessor.run(
-                [
-                    "date",
-                    "end=2014-07-26",
-                    "start=2014-06-26",
-                    "in_range",
-                    "foo",
-                ]
-            ),
-        )
-
-        # invalid syntax - no change
-        self.assertEqual(["date"], self.preprocessor.run(["date"]))
-        self.assertEqual(
-            ["date", "start=2014-06-26"],
-            self.preprocessor.run(["date", "start=2014-06-26"]),
-        )
-        self.assertEqual(
-            ["date", "end=2014-06-26"],
-            self.preprocessor.run(["date", "end=2014-06-26"]),
-        )
-        self.assertEqual(
-            ["date", "start=2014-06-26", "end=2014-07-26"],
-            self.preprocessor.run(
-                ["date", "start=2014-06-26", "end=2014-07-26"]
-            ),
-        )
-        self.assertEqual(
-            ["date", "start=2014-06-26", "end=2014-07-26", "lt"],
-            self.preprocessor.run(
-                ["date", "start=2014-06-26", "end=2014-07-26", "lt"]
-            ),
-        )
-        self.assertEqual(
-            ["date", "start=2014-06-26", "lt", "foo"],
-            self.preprocessor.run(["date", "start=2014-06-26", "lt", "foo"]),
-        )
-        self.assertEqual(
-            ["date", "start=2014-06-26", "end=2014-07-26", "gt", "foo"],
-            self.preprocessor.run(
-                ["date", "start=2014-06-26", "end=2014-07-26", "gt", "foo"]
-            ),
-        )
-        self.assertEqual(
-            ["date", "end=2014-06-26", "gt"],
-            self.preprocessor.run(["date", "end=2014-06-26", "gt"]),
-        )
-        self.assertEqual(
-            ["date", "start=2014-06-26", "in_range", "foo"],
-            self.preprocessor.run(
-                ["date", "start=2014-06-26", "in_range", "foo"]
-            ),
-        )
-        self.assertEqual(
-            ["date", "end=2014-07-26", "in_range"],
-            self.preprocessor.run(["date", "end=2014-07-26", "in_range"]),
-        )
-        self.assertEqual(
-            ["foo", "start=2014-06-26", "gt"],
-            self.preprocessor.run(["foo", "start=2014-06-26", "gt"]),
-        )
-        self.assertEqual(
-            ["foo", "end=2014-06-26", "lt"],
-            self.preprocessor.run(["foo", "end=2014-06-26", "lt"]),
-        )
-
     def testParenthesis(self):
         self.assertEqual(["("], self.preprocessor.run(["("]))
+
         self.assertEqual([")"], self.preprocessor.run([")"]))
+
         self.assertEqual(
             ["(", "(", ")", ")"], self.preprocessor.run(["(", "(", ")", ")"])
         )
+
         self.assertEqual(["(", "(", ")", ")"], self.preprocessor.run(["(())"]))
+
         self.assertEqual(
             ["a", "(", "b", ")", "c"],
             self.preprocessor.run(["a", "(", "b", ")", "c"]),
         )
+
         self.assertEqual(
             ["a", "(", "b", "c", ")", "d"],
             self.preprocessor.run(["a", "(", "b", "c", ")", "d"]),
         )
+
         self.assertEqual(
             ["a", ")", "b", "(", "c"],
             self.preprocessor.run(["a", ")", "b", "(", "c"]),
         )
+
         self.assertEqual(
             ["a", "(", "b", ")", "c"], self.preprocessor.run(["a", "(b)", "c"])
         )
+
         self.assertEqual(
             ["a", "(", "b", ")", "c"], self.preprocessor.run(["a(", "b", ")c"])
         )
+
         self.assertEqual(
             ["a", "(", "b", ")", "c"], self.preprocessor.run(["a(b)c"])
         )
+
         self.assertEqual(
             ["aA", "(", "bB", ")", "cC"], self.preprocessor.run(["aA(bB)cC"])
         )
+
         self.assertEqual(
             ["(", "aA", "(", "bB", ")", "cC", ")"],
             self.preprocessor.run(["(aA(bB)cC)"]),
         )
+
         self.assertEqual(
             ["(", "aA", "(", "(", "bB", ")", "cC", ")"],
             self.preprocessor.run(["(aA(", "(bB)cC)"]),
         )
+
         self.assertEqual(
             ["(", "aA", "(", "(", "(", "bB", ")", "cC", ")"],
             self.preprocessor.run(["(aA(", "(", "(bB)cC)"]),
@@ -2153,30 +2031,87 @@ class DomRuleAddXmlTest(TestCase):
             """,
         )
         self.assertExpressionXml(
-            ["role=master", "#uname", "eq", "node1"],
-            """
+            [f"role={const.PCMK_ROLE_PROMOTED}", "#uname", "eq", "node1"],
+            f"""
 <rsc_location id="location-dummy">
-    <rule id="location-dummy-rule" role="master" score="INFINITY">
+    <rule id="location-dummy-rule" role="{const.PCMK_ROLE_PROMOTED}" score="INFINITY">
+        <expression attribute="#uname" id="location-dummy-rule-expr" operation="eq" value="node1"/>
+    </rule>
+</rsc_location>
+            """,
+            cib_file=empty_cib_new_roles_supported,
+        )
+        self.assertExpressionXml(
+            [f"role={const.PCMK_ROLE_PROMOTED}", "#uname", "eq", "node1"],
+            f"""
+<rsc_location id="location-dummy">
+    <rule id="location-dummy-rule" role="{const.PCMK_ROLE_PROMOTED_LEGACY}" score="INFINITY">
         <expression attribute="#uname" id="location-dummy-rule-expr" operation="eq" value="node1"/>
     </rule>
 </rsc_location>
             """,
         )
         self.assertExpressionXml(
-            ["role=slave", "#uname", "eq", "node1"],
-            """
+            [
+                f"role={str(const.PCMK_ROLE_UNPROMOTED).lower()}",
+                "#uname",
+                "eq",
+                "node1",
+            ],
+            f"""
 <rsc_location id="location-dummy">
-    <rule id="location-dummy-rule" role="slave" score="INFINITY">
+    <rule id="location-dummy-rule" role="{const.PCMK_ROLE_UNPROMOTED_LEGACY}" score="INFINITY">
         <expression attribute="#uname" id="location-dummy-rule-expr" operation="eq" value="node1"/>
     </rule>
 </rsc_location>
             """,
         )
         self.assertExpressionXml(
-            ["score=100", "id=myRule", "role=master", "#uname", "eq", "node1"],
-            """
+            [
+                f"role={str(const.PCMK_ROLE_UNPROMOTED).lower()}",
+                "#uname",
+                "eq",
+                "node1",
+            ],
+            f"""
 <rsc_location id="location-dummy">
-    <rule id="myRule" role="master" score="100">
+    <rule id="location-dummy-rule" role="{const.PCMK_ROLE_UNPROMOTED}" score="INFINITY">
+        <expression attribute="#uname" id="location-dummy-rule-expr" operation="eq" value="node1"/>
+    </rule>
+</rsc_location>
+            """,
+            cib_file=empty_cib_new_roles_supported,
+        )
+        self.assertExpressionXml(
+            [
+                "score=100",
+                "id=myRule",
+                f"role={str(const.PCMK_ROLE_PROMOTED).lower()}",
+                "#uname",
+                "eq",
+                "node1",
+            ],
+            f"""
+<rsc_location id="location-dummy">
+    <rule id="myRule" role="{const.PCMK_ROLE_PROMOTED}" score="100">
+        <expression attribute="#uname" id="myRule-expr" operation="eq" value="node1"/>
+    </rule>
+</rsc_location>
+            """,
+            cib_file=empty_cib_new_roles_supported,
+        )
+        self.assertExpressionXml(
+            [
+                "score=100",
+                "id=myRule",
+                f"role={str(const.PCMK_ROLE_PROMOTED).lower()}",
+                "#uname",
+                "eq",
+                "node1",
+            ],
+            f"""
+<rsc_location id="location-dummy">
+    <rule id="myRule" role="{const.PCMK_ROLE_PROMOTED_LEGACY}" score="100">
         <expression attribute="#uname" id="myRule-expr" operation="eq" value="node1"/>
     </rule>
 </rsc_location>
@@ -2219,7 +2154,7 @@ class DomRuleAddXmlTest(TestCase):
             constraint_el,
             options,
             rule_argv,
-            Version(*utils.getValidateWithVersion(cib_dom)),
+            utils.getValidateWithVersion(cib_dom),
         )
         assert_xml_equal(
             constraint_el.toprettyxml(indent="    "),
@@ -2252,7 +2187,12 @@ class DomRuleAddTest(TestCase):
 
         output, returnVal = pcs(
             self.temp_cib.name,
-            "constraint location dummy1 rule id=MyRule score=100 role=master #uname eq node2".split(),
+            (
+                "constraint location dummy1 rule id=MyRule score=100 role="
+                "{role} #uname eq node2"
+            )
+            .format(role=str(const.PCMK_ROLE_PROMOTED).lower())
+            .split(),
         )
         ac(output, "")
         self.assertEqual(0, returnVal)
@@ -2265,18 +2205,18 @@ class DomRuleAddTest(TestCase):
         self.assertEqual(0, returnVal)
 
         output, returnVal = pcs(
-            self.temp_cib.name, "constraint location show --full".split()
+            self.temp_cib.name, "constraint location config --full".split()
         )
         ac(
             output,
-            """\
+            f"""\
 Location Constraints:
   Resource: dummy1
     Constraint: location-dummy1
       Rule: score=INFINITY (id:location-dummy1-rule)
         Expression: #uname eq node1 (id:location-dummy1-rule-expr)
     Constraint: location-dummy1-1
-      Rule: role=master score=100 (id:MyRule)
+      Rule: role={const.PCMK_ROLE_PROMOTED_PRIMARY} score=100 (id:MyRule)
         Expression: #uname eq node2 (id:MyRule-expr)
     Constraint: location-dummy1-2
       Rule: boolean-op=or score=INFINITY (id:complexRule)
@@ -2293,18 +2233,18 @@ Location Constraints:
         self.assertEqual(0, returnVal)
 
         output, returnVal = pcs(
-            self.temp_cib.name, "constraint location show".split()
+            self.temp_cib.name, "constraint location config".split()
         )
         ac(
             output,
-            """\
+            f"""\
 Location Constraints:
   Resource: dummy1
     Constraint: location-dummy1
       Rule: score=INFINITY
         Expression: #uname eq node1
     Constraint: location-dummy1-1
-      Rule: role=master score=100
+      Rule: role={const.PCMK_ROLE_PROMOTED_PRIMARY} score=100
         Expression: #uname eq node2
     Constraint: location-dummy1-2
       Rule: boolean-op=or score=INFINITY
@@ -2324,17 +2264,13 @@ Location Constraints:
     def test_invalid_score(self):
         output, returnVal = pcs(
             self.temp_cib.name,
-            "constraint location dummy1 rule score=pingd defined pingd".split(),
-        )
-        ac(
-            output,
-            "Warning: invalid score 'pingd', setting score-attribute=pingd "
-            "instead\n",
+            "constraint location dummy1 rule score-attribute=pingd defined pingd".split(),
         )
+        ac(output, "")
         self.assertEqual(0, returnVal)
 
         output, returnVal = pcs(
-            self.temp_cib.name, "constraint location show --full".split()
+            self.temp_cib.name, "constraint location config --full".split()
         )
         ac(
             output,
@@ -2384,7 +2320,15 @@ Location Constraints:
             self.temp_cib.name,
             "constraint location dummy1 rule role=foo #uname eq node1".split(),
         )
-        ac(output, "Error: invalid role 'foo', use 'master' or 'slave'\n")
+        ac(
+            output,
+            "Error: invalid role 'foo', use {}\n".format(
+                format_list_custom_last_separator(
+                    const.PCMK_ROLES_PROMOTED + const.PCMK_ROLES_UNPROMOTED,
+                    " or ",
+                )
+            ),
+        )
         self.assertEqual(1, returnVal)
 
         output, returnVal = pcs(
@@ -2406,7 +2350,7 @@ Location Constraints:
         self.assertEqual(1, returnVal)
 
         output, returnVal = pcs(
-            self.temp_cib.name, "constraint location show --full".split()
+            self.temp_cib.name, "constraint location config --full".split()
         )
         ac(output, "Location Constraints:\n")
         self.assertEqual(0, returnVal)
@@ -2419,7 +2363,7 @@ Location Constraints:
         self.assertEqual(0, returnVal)
 
         output, returnVal = pcs(
-            self.temp_cib.name, "constraint location show --full".split()
+            self.temp_cib.name, "constraint location config --full".split()
         )
         ac(
             output,
