@@ -1,16 +1,14 @@
 import difflib
 import doctest
 import re
+
 from lxml.doctestcompare import LXMLOutputChecker
 from lxml.etree import LXML_VERSION
 
 from pcs.common import reports
 from pcs.lib.errors import LibraryError
 
-# pylint: disable=invalid-name, no-self-use
-
-# cover python2 vs. python3 differences
-_re_object_type = type(re.compile(""))
+# pylint: disable=invalid-name
 
 
 def prepare_diff(first, second):
@@ -43,12 +41,7 @@ def start_tag_error_text():
     return msg
 
 
-def console_report(*lines):
-    # after lines append last new line
-    return "\n".join(lines + ("",))
-
-
-class AssertPcsMixin:
+class AssertPcsMixinOld:
     """Run pcs command and assert its result"""
 
     def assert_pcs_success_all(self, command_list):
@@ -129,15 +122,7 @@ class AssertPcsMixin:
         if len(specified_stdout) > 1:
             raise Exception(msg + ", both specified")
 
-        stdout, pcs_returncode = self.pcs_runner.run(command)
-        self.assertEqual(
-            returncode,
-            pcs_returncode,
-            (
-                'Expected return code "{0}", but was "{1}"'
-                + "\ncommand: {2}\nstdout:\n{3}"
-            ).format(returncode, pcs_returncode, command, stdout),
-        )
+        stdout = self.assert_pcs_success_ignore_output(command, returncode)
         message_template = (
             "{reason}\ncommand: {cmd}\ndiff is (expected is 2nd):\n{diff}"
             + "\nFull stdout:\n{stdout}"
@@ -156,7 +141,7 @@ class AssertPcsMixin:
                     )
                 )
         elif stdout_regexp:
-            if not isinstance(stdout_regexp, _re_object_type):
+            if not isinstance(stdout_regexp, re.Pattern):
                 stdout_regexp = re.compile(stdout_regexp)
             if not stdout_regexp.search(stdout):
                 self.fail(
@@ -189,12 +174,26 @@ class AssertPcsMixin:
                     ),
                 )
 
+    def assert_pcs_success_ignore_output(self, command, returncode=0):
+        stdout, retval_actual = self.pcs_runner.run(command)
+
+        self.assertEqual(
+            returncode,
+            retval_actual,
+            (
+                f"Expected return code '{returncode}' but was '{retval_actual}'\n"
+                f"** command: {command}\n"
+                f"** stdout:\n{stdout}\n"
+            ),
+        )
+        return stdout
+
     def __prepare_output(self, output):
-        if isinstance(output, list):
-            return console_report(*output)
-        return output
+        # pylint: disable=no-self-use
+        return "\n".join(output + [""]) if isinstance(output, list) else output
 
     def __prepare_regexp_flags(self, flags):
+        # pylint: disable=no-self-use
         # python2 has different flags than python3
         possible_flags = [
             "ASCII",
@@ -214,6 +213,263 @@ class AssertPcsMixin:
         return sorted(used_flags)
 
 
+class AssertPcsMixin:
+    """Run pcs command and assert its result"""
+
+    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-locals
+
+    def assert_pcs_success_all(self, command_list):
+        for command in command_list:
+            stdout, stderr, retval = self.pcs_runner.run(command)
+            if retval != 0:
+                raise AssertionError(
+                    f"Command '{command}' did not succeed\n"
+                    f"** return code: {retval}\n"
+                    f"** stdout:\n{stdout}\n"
+                    f"** stderr:\n{stderr}\n"
+                )
+
+    def assert_pcs_success(
+        self,
+        command,
+        stdout_full=None,
+        stdout_start=None,
+        stdout_regexp=None,
+        stderr_full=None,
+        stderr_start=None,
+        stderr_regexp=None,
+        despace=False,
+    ):
+        # It is common that successful commands don't print anything, so we
+        # default stdout and stderr to an empty string if not specified
+        # otherwise.
+        self.assert_pcs_result(
+            command,
+            stdout_full=self.__default_output_to_empty_str(
+                stdout_full, stdout_start, stdout_regexp
+            ),
+            stdout_start=stdout_start,
+            stdout_regexp=stdout_regexp,
+            stderr_full=self.__default_output_to_empty_str(
+                stderr_full, stderr_start, stderr_regexp
+            ),
+            stderr_start=stderr_start,
+            stderr_regexp=stderr_regexp,
+            returncode=0,
+            despace=despace,
+        )
+
+    def assert_pcs_fail(
+        self,
+        command,
+        stderr_full=None,
+        stderr_start=None,
+        stderr_regexp=None,
+        stdout_full=None,
+        stdout_start=None,
+        stdout_regexp=None,
+    ):
+        # It is common that failed commands don't print anything to stdout, so
+        # we default stdout to an empty string if not specified otherwise.
+        self.assert_pcs_result(
+            command,
+            stdout_full=self.__default_output_to_empty_str(
+                stdout_full, stdout_start, stdout_regexp
+            ),
+            stdout_start=stdout_start,
+            stdout_regexp=stdout_regexp,
+            stderr_full=stderr_full,
+            stderr_start=stderr_start,
+            stderr_regexp=stderr_regexp,
+            returncode=1,
+        )
+
+    def assert_pcs_fail_regardless_of_force(
+        self,
+        command,
+        stderr_full=None,
+        stderr_start=None,
+        stderr_regexp=None,
+        stdout_full=None,
+        stdout_start=None,
+        stdout_regexp=None,
+    ):
+        self.assert_pcs_fail(
+            command,
+            stdout_full=stdout_full,
+            stdout_start=stdout_start,
+            stdout_regexp=stdout_regexp,
+            stderr_full=stderr_full,
+            stderr_start=stderr_start,
+            stderr_regexp=stderr_regexp,
+        )
+        self.assert_pcs_fail(
+            command + ["--force"],
+            stdout_full=stdout_full,
+            stdout_start=stdout_start,
+            stdout_regexp=stdout_regexp,
+            stderr_full=stderr_full,
+            stderr_start=stderr_start,
+            stderr_regexp=stderr_regexp,
+        )
+
+    def assert_pcs_result(
+        self,
+        command,
+        stdout_full=None,
+        stdout_start=None,
+        stdout_regexp=None,
+        stderr_full=None,
+        stderr_start=None,
+        stderr_regexp=None,
+        returncode=0,
+        despace=False,
+    ):
+
+        self.__check_output_specified(
+            stdout_full, stdout_start, stdout_regexp, "stdout"
+        )
+        self.__check_output_specified(
+            stderr_full, stderr_start, stderr_regexp, "stderr"
+        )
+
+        stdout_actual, stderr_actual = self.assert_pcs_success_ignore_output(
+            command, returncode
+        )
+
+        message_template = (
+            f"{{reason}}\n** Command: {command}\n** {{detail}}\n"
+            f"** Retval: {returncode}\n"
+            f"** Full stdout:\n{stdout_actual}\n"
+            f"** Full stderr:\n{stderr_actual}\n"
+        )
+        self.__fail_on_unexpected_output(
+            stdout_actual,
+            stdout_full,
+            stdout_start,
+            stdout_regexp,
+            despace,
+            "stdout",
+            message_template,
+        )
+
+        self.__fail_on_unexpected_output(
+            stderr_actual,
+            stderr_full,
+            stderr_start,
+            stderr_regexp,
+            despace,
+            "stderr",
+            message_template,
+        )
+
+    def assert_pcs_success_ignore_output(self, command, returncode=0):
+        stdout, stderr, retval_actual = self.pcs_runner.run(command)
+
+        self.assertEqual(
+            returncode,
+            retval_actual,
+            (
+                f"Expected return code '{returncode}' but was '{retval_actual}'\n"
+                f"** command: {command}\n"
+                f"** stdout:\n{stdout}\n"
+                f"** stderr:\n{stderr}\n"
+            ),
+        )
+        return stdout, stderr
+
+    @staticmethod
+    def __default_output_to_empty_str(output_full, output_start, output_regexp):
+        if (
+            output_start is None
+            and output_full is None
+            and output_regexp is None
+        ):
+            return ""
+        return output_full
+
+    @staticmethod
+    def __check_output_specified(
+        output_full, output_start, output_regexp, label
+    ):
+        msg = (
+            f"Please specify exactly one: {label}_full or {label}_start or "
+            f"{label}_regexp"
+        )
+        specified_output = [
+            output
+            for output in (output_full, output_start, output_regexp)
+            if output is not None
+        ]
+        if not specified_output:
+            raise Exception(msg + ", none specified")
+        if len(specified_output) > 1:
+            raise Exception(msg + ", more than one specified")
+
+    def __fail_on_unexpected_output(
+        self,
+        output_actual,
+        output_full,
+        output_start,
+        output_regexp,
+        despace,
+        label,
+        message_template,
+    ):
+        if output_start:
+            expected_start = self.__prepare_output(output_start)
+            if not output_actual.startswith(expected_start):
+                diff = prepare_diff(
+                    output_actual[: len(expected_start)], expected_start
+                )
+                raise AssertionError(
+                    message_template.format(
+                        reason=f"{label} does not start as expected",
+                        detail=f"diff is (expected is 2nd):\n{diff}\n",
+                    )
+                )
+            return
+
+        if output_regexp:
+            if not isinstance(output_regexp, re.Pattern):
+                output_regexp = re.compile(output_regexp)
+            if not output_regexp.search(output_actual):
+                flags = ", ".join(
+                    self.__prepare_regexp_flags(output_regexp.flags)
+                )
+                raise AssertionError(
+                    message_template.format(
+                        reason=f"{label} does not match the expected regexp",
+                        detail=(
+                            f"regexp:\n{output_regexp.pattern}\n"
+                            f"regexp flags: {flags}\n"
+                        ),
+                    )
+                )
+            return
+
+        expected_full = self.__prepare_output(output_full)
+        if (despace and _despace(output_actual) != _despace(expected_full)) or (
+            not despace and output_actual != expected_full
+        ):
+            diff = prepare_diff(output_actual, expected_full)
+            raise AssertionError(
+                message_template.format(
+                    reason=f"{label} is not as expected",
+                    detail=f"diff is (expected is 2nd):\n{diff}\n",
+                ),
+            )
+
+    @staticmethod
+    def __prepare_output(output):
+        return "\n".join(output + [""]) if isinstance(output, list) else output
+
+    @staticmethod
+    def __prepare_regexp_flags(used_flags):
+        return sorted([flag.name for flag in re.RegexFlag if used_flags & flag])
+
+
 class ExtendedAssertionsMixin:
     def assert_raises(
         self, expected_exception, callable_obj, property_dict=None
@@ -231,13 +487,13 @@ class ExtendedAssertionsMixin:
             for prop, value in property_dict.items():
                 try:
                     self.assertEqual(value, getattr(e, prop))
-                except AttributeError as e:
+                except AttributeError as exc:
                     raise AssertionError(
                         "Property {property} doesn't exist in exception"
                         " {exception}".format(
-                            property=prop, exception=e.__class__.__name__
+                            property=prop, exception=exc.__class__.__name__
                         )
-                    ) from e
+                    ) from exc
 
 
 def assert_xml_equal(expected_xml, got_xml, context_explanation=""):
@@ -260,6 +516,7 @@ def assert_xml_equal(expected_xml, got_xml, context_explanation=""):
 SEVERITY_SHORTCUTS = {
     reports.ReportItemSeverity.INFO: "I",
     reports.ReportItemSeverity.WARNING: "W",
+    reports.ReportItemSeverity.DEPRECATION: "DW",
     reports.ReportItemSeverity.ERROR: "E",
     reports.ReportItemSeverity.DEBUG: "D",
 }
@@ -272,7 +529,7 @@ def _format_report_item_info(info):
 
 
 def _expected_report_item_format(report_item_expectation):
-    return "{0} {1} {{{2}}} ! {3}".format(
+    return "{0} {1} {{{2}}} ! {3} {4}".format(
         SEVERITY_SHORTCUTS.get(
             report_item_expectation[0], report_item_expectation[0]
         ),
@@ -281,6 +538,9 @@ def _expected_report_item_format(report_item_expectation):
         report_item_expectation[3]
         if len(report_item_expectation) > 3
         else None,
+        report_item_expectation[4]
+        if len(report_item_expectation) > 4
+        else None,
     )
 
 
@@ -307,6 +567,9 @@ def assert_report_item_equal(real_report_item, report_item_info):
                         None
                         if len(report_item_info) < 4
                         else report_item_info[3],
+                        None
+                        if len(report_item_info) < 5
+                        else report_item_info[4],
                     )
                 ),
                 _format_report_item(real_report_item),
@@ -366,6 +629,12 @@ def assert_report_item_list_equal(
             expected_report_info_list, real_report_item
         )
         if found_report_info is None:
+            if (
+                real_report_item.severity.level
+                == reports.ReportItemSeverity.DEBUG
+            ):
+                # ignore debug report items not specified as expected
+                continue
             raise _unexpected_report_given(
                 remaining_expected_report_info_list,
                 expected_report_info_list,
@@ -385,8 +654,8 @@ def assert_report_item_list_equal(
             )
 
         raise AssertionError(
-            "\nExpected LibraryError is missing\n{0}\n\n{1}\n\n{2}".format(
-                "{0}\n".format(hint) if hint else "",
+            "\nReport lists doesn't match{0}\n\n{1}\n\n{2}".format(
+                "\n{0}".format(hint) if hint else "",
                 format_items("expected", expected_report_info_list),
                 format_items("real", real_report_item_list),
             )
@@ -418,9 +687,8 @@ def __report_item_equal(real_report_item, report_item_info):
             report_dto.severity.force_code
             == (None if len(report_item_info) < 4 else report_item_info[3])
         )
-        and
-        # TODO: add proper check for context once it will be used
-        report_dto.context is None
+        and report_dto.context
+        == (report_item_info[4] if len(report_item_info) >= 5 else None)
     )
 
 
