@@ -4,11 +4,17 @@ import os
 import os.path
 import sys
 from functools import partial
-
 from unittest import mock
 
+from pcs.common.file import RawFile
+from pcs.common.node_communicator import NodeCommunicatorFactory
+from pcs.lib.env import LibraryEnvironment
+
 from pcs_test.tools import fixture
-from pcs_test.tools.assertions import assert_raise_library_error, prepare_diff
+from pcs_test.tools.assertions import (
+    assert_raise_library_error,
+    prepare_diff,
+)
 from pcs_test.tools.case_analysis import test_failed
 from pcs_test.tools.command_env import spy
 from pcs_test.tools.command_env.calls import Queue as CallQueue
@@ -34,20 +40,16 @@ from pcs_test.tools.command_env.mock_runner import Runner
 from pcs_test.tools.command_env.mock_service_manager import ServiceManagerMock
 from pcs_test.tools.custom_mock import MockLibraryReportProcessor
 
-from pcs.common.file import RawFile
-from pcs.common.node_communicator import NodeCommunicatorFactory
-from pcs.lib.env import LibraryEnvironment
-
 patch_lib_env = partial(mock.patch.object, LibraryEnvironment)
 
 
 def patch_env(call_queue, config, init_env, is_systemd=True):
     # pylint: disable=too-many-locals
-    # It is mandatory to patch some env objects/methods. It is ok when command
-    # does not use this objects/methods and specify no call for it. But it would
-    # be a problem when the test succeded because the live call respond
-    # correctly by accident. Such test would fails on different machine (with
-    # another live environment)
+    # It is mandatory to patch some env objects/methods. It is ok when a
+    # command does not use these objects/methods and specifies no calls for
+    # them. But it would be a problem when a test succeeds thanks to a live
+    # call responded correctly by accident. Such tests would fail on a
+    # different machine (with an other live environment).
 
     orig_cmd_runner = init_env.cmd_runner
     get_node_communicator = init_env.get_node_communicator
@@ -86,7 +88,7 @@ def patch_env(call_queue, config, init_env, is_systemd=True):
         ),
     ]
     if is_systemd:
-        # In most test cases we don't care about underlaying init system. But
+        # In most test cases we don't care about underlying init system. But
         # some tests actually test different behavior based on init
         # system. This will cause pcs.lib.services.is_systemd(<instance of
         # ServiceManagerMock>) to return True because we replace SystemdDriver
@@ -198,6 +200,11 @@ class EnvAssistant:
             self.__unpatch()
 
         if test_failed(current_test):
+            # NOTE: This detection of failed test does not work when tests are
+            # running in parallel by python concurrencytest module with Python
+            # version 3.11. This cause that the reports for failed tests will
+            # be a little more confusing
+            #
             # We have already got the message that main test failed. There is
             # a high probability that something remains in reports or in the
             # queue etc. But it is only consequence of the main test fail. And
@@ -306,7 +313,7 @@ class EnvAssistant:
     def __assert_environment_created(self):
         if not hasattr(self, "_env"):
             raise AssertionError(
-                "LibraryEnvironment was not created in EnvAssitant."
+                "LibraryEnvironment was not created in EnvAssistant."
                 " Have you called method get_env?"
             )
 
