@@ -15,10 +15,12 @@ DEFAULT_WAIT_TIMEOUT = 10
 WAIT_TIMEOUT_EXPIRED_RETURNCODE = 124
 AGENT_FILENAME_MAP = {
     "ocf:heartbeat:Dummy": "resource_agent_ocf_heartbeat_dummy.xml",
+    "ocf:pacemaker:Dummy": "resource_agent_ocf_pacemaker_dummy.xml",
     "ocf:pacemaker:remote": "resource_agent_ocf_pacemaker_remote.xml",
     "ocf:heartbeat:IPaddr2": "resource_agent_ocf_heartbeat_ipaddr2.xml",
     "ocf:pacemaker:booth-site": "resource_agent_ocf_pacemaker_booth-site.xml",
     "ocf:pacemaker:Stateful": "resource_agent_ocf_pacemaker_stateful_ocf_1.1.xml",
+    "systemd:chronyd": "resource_agent_systemd_chronyd.xml",
     "stonith:fence_unfencing": "stonith_agent_fence_unfencing.xml",
 }
 
@@ -440,9 +442,10 @@ class PcmkShortcuts:
             instead=instead,
         )
 
-    def load_fenced_metadata(
+    def load_fake_agent_metadata(
         self,
-        name="runner.pcmk.load_fenced_metadata",
+        name="runner.pcmk.load_fake_agent_metadata",
+        agent_name="pacemaker-fenced",
         stdout=None,
         stderr="",
         returncode=0,
@@ -450,24 +453,33 @@ class PcmkShortcuts:
         before=None,
     ):
         """
-        Create a call for loading fenced metadata - additional fence options
+        Create a call for loading fake agent metadata - usually metadata
+        provided by pacemaker daemon
 
         string name -- the key of this call
-        string stdout -- fenced stdout, default metadata if None
-        string stderr -- fenced stderr
-        int returncode -- fenced returncode
+        string agent_name -- name of the fake agent
+        string stdout -- fake agent stdout, default metadata if None
+        string stderr -- fake agent stderr
+        int returncode -- fake agent returncode
         string instead -- the key of a call instead of which this new call is to
             be placed
         string before -- the key of a call before which this new call is to be
             placed
         """
+        name_to_metadata_file = {
+            "pacemaker-based": "based_metadata.xml",
+            "pacemaker-controld": "controld_metadata.xml",
+            "pacemaker-fenced": "fenced_metadata.xml",
+            "pacemaker-schedulerd": "schedulerd_metadata.xml",
+        }
         if stdout is None:
-            with open(rc("fenced_metadata.xml")) as a_file:
+            with open(rc(name_to_metadata_file[agent_name])) as a_file:
                 stdout = a_file.read()
+        agent_path = settings.__dict__[agent_name.replace("-", "_")]
         self.__calls.place(
             name,
             RunnerCall(
-                [settings.pacemaker_fenced, "metadata"],
+                [agent_path, "metadata"],
                 stdout=stdout,
                 stderr=stderr,
                 returncode=returncode,
@@ -1007,3 +1019,100 @@ class PcmkShortcuts:
                 returncode=returncode,
             ),
         )
+
+    def resource_agent_self_validation(
+        self,
+        attributes,
+        standard="ocf",
+        provider="heartbeat",
+        agent_type="Dummy",
+        returncode=0,
+        output=None,
+        stdout="",
+        instead=None,
+        env=None,
+        name="runner.pcmk.resource_agent_self_validation",
+    ):
+        # pylint: disable=too-many-locals
+        if output and stdout:
+            raise AssertionError("Cannot specify both output and stdout")
+        cmd = [
+            "crm_resource",
+            "--validate",
+            "--output-as",
+            "xml",
+            "--class",
+            standard,
+            "--agent",
+            agent_type,
+        ]
+        if provider:
+            cmd.extend(["--provider", provider])
+        for key, value in sorted(attributes.items()):
+            cmd.extend(["--option", f"{key}={value}"])
+        if output is not None:
+            cmd_str = " ".join(cmd)
+            provider_str = f' provider="{provider}"' if provider else ""
+            stdout = f"""
+            <pacemaker-result api-version="2.15" request="{cmd_str}">
+              <resource-agent-action action="validate" class="{standard}" type="{agent_type}"{provider_str}>
+                <overrides/>
+                <agent-status code="5" message="not installed" execution_code="0" execution_message="complete" reason="environment is invalid, resource considered stopped"/>
+                <command code="5">
+                  {output}
+                </command>
+              </resource-agent-action>
+              <status code="5" message="Not installed">
+                <errors>
+                  <error>crm_resource: Error performing operation: Not installed</error>
+                </errors>
+              </status>
+            </pacemaker-result>
+            """
+        self.__calls.place(
+            name,
+            RunnerCall(cmd, stdout=stdout, returncode=returncode, env=env),
+            instead=instead,
+        )
+
+    def stonith_agent_self_validation(
+        self,
+        attributes,
+        agent,
+        returncode=0,
+        output=None,
+        stdout="",
+        instead=None,
+        env=None,
+        name="runner.pcmk.stonith_agent_self_validation",
+    ):
+        # pylint: disable=too-many-locals
+        if output and stdout:
+            raise AssertionError("Cannot specify both output and stdout")
+        cmd = [
+            "stonith_admin",
+            "--validate",
+            "--output-as",
+            "xml",
+            "--agent",
+            agent,
+        ]
+        for key, value in sorted(attributes.items()):
+            cmd.extend(["--option", f"{key}={value}"])
+        if output is not None:
+            cmd_str = " ".join(cmd)
+            stdout = f"""
+            <pacemaker-result api-version="2.22" request="{cmd_str}">
+              <validate agent="{agent}" valid="false">
+                <command code="-201">
+                  {output}
+                </command>
+              </validate>
+              <status code="1" message="Error occurred"/>
+            </pacemaker-result>
+            """
+        self.__calls.place(
+            name,
+            RunnerCall(cmd, stdout=stdout, returncode=returncode, env=env),
+            instead=instead,
+        )
