@@ -1,7 +1,7 @@
 # Wrapper for PCS command
 
 require 'etc'
-require 'open4'
+require 'childprocess'
 require 'shellwords'
 require 'cgi'
 require 'net/http'
@@ -9,9 +9,10 @@ require 'net/https'
 require 'uri'
 require 'json'
 require 'fileutils'
-require 'backports'
+require 'backports/latest'
 require 'base64'
 require 'ethon'
+require 'openssl'
 
 require 'config.rb'
 require 'cfgsync.rb'
@@ -19,6 +20,7 @@ require 'corosyncconf.rb'
 require 'resource.rb'
 require 'cluster_entity.rb'
 require 'auth.rb'
+require 'settings.rb'
 
 class NotImplementedException < NotImplementedError
 end
@@ -42,12 +44,12 @@ end
 def add_fence_level(auth_user, level, devices, node, remove = false)
   if not remove
     stdout, stderr, retval = run_cmd(
-      auth_user, PCS, "stonith", "level", "add", level, node, devices
+      auth_user, PCS, "--", "stonith", "level", "add", level, node, devices
     )
     return retval,stdout, stderr
   else
     stdout, stderr, retval = run_cmd(
-      auth_user, PCS, "stonith", "level", "remove", level, node, devices
+      auth_user, PCS, "--", "stonith", "level", "remove", level, "target", node, "stonith", devices
     )
     return retval,stdout, stderr
   end
@@ -55,22 +57,23 @@ end
 
 def add_node_attr(auth_user, node, key, value)
   stdout, stderr, retval = run_cmd(
-    auth_user, PCS, "node", "attribute", node, key.to_s + '=' + value.to_s
+    auth_user, PCS, "--", "node", "attribute", node, key.to_s + '=' + value.to_s
   )
   return retval
 end
 
 def add_meta_attr(auth_user, resource, key, value)
   cmd = ["resource", "meta", resource, key.to_s + "=" + value.to_s]
+  flags = []
   if key.to_s == "remote-node"
     # --force is a workaround for:
     # 1) Error: this command is not sufficient for create guest node, use 'pcs
     # cluster node add-guest', use --force to override
     # 2) Error: this command is not sufficient for remove guest node, use 'pcs
     # cluster node remove-guest', use --force to override
-    cmd << "--force"
+    flags << "--force"
   end
-  stdout, stderr, retval = run_cmd(auth_user, PCS, *cmd)
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *cmd)
   return retval
 end
 
@@ -87,17 +90,18 @@ def add_location_constraint(
     nodescore = node + "=" + score
   end
 
-  cmd = [PCS, "constraint", "location", resource, "prefers", nodescore]
-  cmd << '--force' if force
+  cmd = ["constraint", "location", resource, "prefers", nodescore]
+  flags = []
+  flags << '--force' if force
 
-  stdout, stderr, retval = run_cmd(auth_user, *cmd)
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *cmd)
   return retval, stderr.join(' ')
 end
 
 def add_location_constraint_rule(
   auth_user, resource, rule, score, force=false
 )
-  cmd = [PCS, "constraint", "location", resource, "rule"]
+  cmd = ["constraint", "location", resource, "rule"]
   if score != ''
     if is_score(score.upcase)
       cmd << "score=#{score.upcase}"
@@ -106,8 +110,9 @@ def add_location_constraint_rule(
     end
   end
   cmd.concat(rule.shellsplit())
-  cmd << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *cmd)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *cmd)
   return retval, stderr.join(' ')
 end
 
@@ -115,59 +120,60 @@ def add_order_constraint(
     auth_user, resourceA, resourceB, actionA, actionB, score, symmetrical=true,
     force=false
 )
-  sym = symmetrical ? "symmetrical" : "nonsymmetrical"
-  if score != ""
-    score = "score=" + score
-  end
   command = [
-    PCS, "constraint", "order", actionA, resourceA, "then", actionB, resourceB,
-    score, sym
+    "constraint", "order", actionA, resourceA, "then", actionB, resourceB
   ]
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  command << "score=" + score if score != ""
+  command << (symmetrical ? "symmetrical" : "nonsymmetrical")
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
 def add_order_set_constraint(auth_user, resource_set_list, force=false)
-  command = [PCS, "constraint", "order"]
+  command = ["constraint", "order"]
   resource_set_list.each { |resource_set|
     command << "set"
     command.concat(resource_set)
   }
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
 def add_colocation_set_constraint(auth_user, resource_set_list, force=false)
-  command = [PCS, "constraint", "colocation"]
+  command = ["constraint", "colocation"]
   resource_set_list.each { |resource_set|
     command << "set"
     command.concat(resource_set)
   }
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
 def add_ticket_constraint(
     auth_user, ticket, resource_id, role, loss_policy, force=false
 )
-  command = [PCS, "constraint", "ticket", "add", ticket]
+  command = ["constraint", "ticket", "add", ticket]
   if role
     command << role
   end
   command << resource_id
   command << 'loss-policy=' + loss_policy unless loss_policy.strip().empty?()
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
 def add_ticket_set_constraint(
   auth_user, ticket, loss_policy, resource_set_list, force=false
 )
-  command = [PCS, 'constraint', 'ticket']
+  command = ['constraint', 'ticket']
   resource_set_list.each { |resource_set|
     command << 'set'
     command.concat(resource_set)
@@ -175,8 +181,9 @@ def add_ticket_set_constraint(
   command << 'setoptions'
   command << 'ticket=' + ticket
   command << 'loss-policy=' + loss_policy unless loss_policy.strip().empty?()
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
@@ -187,16 +194,17 @@ def add_colocation_constraint(
     score = "INFINITY"
   end
   command = [
-    PCS, "constraint", "colocation", "add", resourceA, resourceB, score
+    "constraint", "colocation", "add", resourceA, "with", resourceB, score
   ]
-  command << '--force' if force
-  stdout, stderr, retval = run_cmd(auth_user, *command)
+  flags = []
+  flags << '--force' if force
+  stdout, stderr, retval = run_cmd(auth_user, PCS, *flags, "--", *command)
   return retval, stderr.join(' ')
 end
 
 def remove_constraint(auth_user, constraint_id)
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "constraint", "remove", constraint_id
+    auth_user, PCS, "--", "constraint", "remove", constraint_id
   )
   $logger.info stdout
   return retval
@@ -204,14 +212,14 @@ end
 
 def remove_constraint_rule(auth_user, rule_id)
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "constraint", "rule", "remove", rule_id
+    auth_user, PCS, "--", "constraint", "rule", "remove", rule_id
   )
   $logger.info stdout
   return retval
 end
 
 def add_acl_role(auth_user, name, description)
-  cmd = [PCS, "acl", "role", "create", name.to_s]
+  cmd = [PCS, "--", "acl", "role", "create", name.to_s]
   if description.to_s != ""
     cmd << "description=#{description.to_s}"
   end
@@ -224,7 +232,7 @@ end
 
 def add_acl_permission(auth_user, acl_role_id, perm_type, xpath_id, query_id)
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "acl", "permission", "add", acl_role_id.to_s, perm_type.to_s,
+    auth_user, PCS, "--", "acl", "permission", "add", acl_role_id.to_s, perm_type.to_s,
     xpath_id.to_s, query_id.to_s
   )
   if retval != 0
@@ -240,7 +248,7 @@ end
 def add_acl_usergroup(auth_user, acl_role_id, user_group, name)
   if (user_group == "user") or (user_group == "group")
     stdout, stderr, retval = run_cmd(
-      auth_user, PCS, "acl", user_group, "create", name.to_s, acl_role_id.to_s
+      auth_user, PCS, "--", "acl", user_group, "create", name.to_s, acl_role_id.to_s
     )
     if retval == 0
       return ""
@@ -251,7 +259,7 @@ def add_acl_usergroup(auth_user, acl_role_id, user_group, name)
     end
   end
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "acl", "role", "assign",
+    auth_user, PCS, "--", "acl", "role", "assign",
     acl_role_id.to_s, user_group, name.to_s
   )
   if retval != 0
@@ -266,7 +274,7 @@ end
 
 def remove_acl_permission(auth_user, acl_perm_id)
   stdout, stderror, retval = run_cmd(
-    auth_user, PCS, "acl", "permission", "delete", acl_perm_id.to_s
+    auth_user, PCS, "--", "acl", "permission", "delete", acl_perm_id.to_s
   )
   if retval != 0
     if stderror.empty?
@@ -281,13 +289,13 @@ end
 def remove_acl_usergroup(auth_user, role_id, usergroup_id, user_or_group)
   if ['user', 'group'].include?(user_or_group)
     stdout, stderror, retval = run_cmd(
-      auth_user, PCS, "acl", "role", "unassign", role_id.to_s, user_or_group,
-      usergroup_id.to_s, "--autodelete"
+      auth_user, PCS, "--autodelete", "--", "acl", "role", "unassign", role_id.to_s, user_or_group,
+      usergroup_id.to_s
     )
   else
     stdout, stderror, retval = run_cmd(
-      auth_user, PCS, "acl", "role", "unassign", role_id.to_s,
-      usergroup_id.to_s, "--autodelete"
+      auth_user, PCS, "--autodelete", "--", "acl", "role", "unassign", role_id.to_s,
+      usergroup_id.to_s
     )
   end
   if retval != 0
@@ -430,8 +438,8 @@ end
 
 def _transform_data(data)
   # Converts data in a way that URI.encode_www_form method will encode it
-  # corectly. If an arrray is passed as value to encode_www_form, then parser of
-  # webbrick will use only last value.
+  # correctly. If an array is passed as a value to encode_www_form, then parser
+  # of webrick will use only the last value.
   new_data = []
   data.each { |key, val|
     if val.kind_of?(Array)
@@ -501,8 +509,11 @@ def send_request(
   })
   req.compose_header('Expect', '')
   return_code = req.perform
+  response_code = req.response_code
+  response_body = req.response_body
+  req.cleanup
   if return_code == :ok
-    return req.response_code, req.response_body
+    return response_code, response_body
   else
     if is_proxy_set(ENV)
       $logger.warn(
@@ -527,16 +538,6 @@ def is_proxy_set(env_var_hash)
   return false
 end
 
-def get_current_node_name()
-  stdout, stderror, retval = run_cmd(
-    PCSAuth.getSuperuserAuth(), CRM_NODE, "-n"
-  )
-  if retval == 0 and stdout.length > 0
-    return stdout[0].chomp()
-  end
-  return ""
-end
-
 def get_local_node_id()
   out, errout, retval = run_cmd(
     PCSAuth.getSuperuserAuth(),
@@ -582,7 +583,7 @@ def get_nodes_status()
   pacemaker_standby = []
   in_pacemaker = false
   stdout, stderr, retval = run_cmd(
-    PCSAuth.getSuperuserAuth(), PCS, "status", "nodes", "both"
+    PCSAuth.getSuperuserAuth(), PCS, "--", "status", "nodes", "both"
   )
   stdout.each {|l|
     l = l.chomp
@@ -624,55 +625,29 @@ def get_nodes_status()
   }
 end
 
-def get_resource_agents_avail(auth_user, params)
-  code, result = send_cluster_request_with_token(
-    auth_user, params[:cluster], 'get_avail_resource_agents'
-  )
-  return [] if 200 != code
-  begin
-    ra = JSON.parse(result)
-    if (ra["noresponse"] == true) or (ra["notauthorized"] == "true") or (ra["notoken"] == true) or (ra["pacemaker_not_running"] == true)
-      return []
-    else
-      return ra.keys
-    end
-  rescue JSON::ParserError
-    return []
-  end
-end
-
-def get_stonith_agents_avail(auth_user, params)
-  code, result = send_cluster_request_with_token(
-    auth_user, params[:cluster], 'get_avail_fence_agents'
-  )
-  return {} if 200 != code
-  begin
-    sa = JSON.parse(result)
-    if (sa["noresponse"] == true) or (sa["notauthorized"] == "true") or (sa["notoken"] == true) or (sa["pacemaker_not_running"] == true)
-      return {}
-    else
-      return sa
-    end
-  rescue JSON::ParserError
-    return {}
-  end
-end
-
-def get_cluster_name()
+def get_cluster_name_and_uuid()
   if has_corosync_conf()
     corosync_conf = CorosyncConf::parse_string(
       Cfgsync::CorosyncConf.from_file().text()
     )
-    # mimic corosync behavior - the last cluster_name found is used
+    # mimic corosync behavior - the last value is used
     cluster_name = ''
+    cluster_uuid = ''
     corosync_conf.sections('totem').each { |totem|
       totem.attributes('cluster_name').each { |attrib|
         cluster_name = attrib[1]
       }
+      totem.attributes('cluster_uuid').each { |attrib|
+        cluster_uuid = attrib[1]
+      }
     }
-    return cluster_name
+    return cluster_name, cluster_uuid
   end
-  return ''
+  return '', ''
+end
+
+def get_cluster_name()
+  return get_cluster_name_and_uuid()[0]
 end
 
 def get_node_attributes(auth_user, cib_dom=nil)
@@ -688,11 +663,11 @@ def get_node_attributes(auth_user, cib_dom=nil)
     node_attrs[node] ||= []
     node_attrs[node] << {
       :id => e.attributes['id'],
-      :key => e.attributes['name'],
+      :name => e.attributes['name'],
       :value => e.attributes['value']
     }
   }
-  node_attrs.each { |_, val| val.sort_by! { |obj| obj[:key] }}
+  node_attrs.each { |_, val| val.sort_by! { |obj| obj[:name] }}
   return node_attrs
 end
 
@@ -784,17 +759,19 @@ def get_acls(auth_user, cib_dom=nil)
 end
 
 def enable_cluster(auth_user, all)
-  cmd = [PCS, "cluster", "enable"]
-  cmd << '--all' if all == '1'
-  stdout, stderror, retval = run_cmd(auth_user, *cmd)
+  cmd = ["cluster", "enable"]
+  flags = []
+  flags << '--all' if all == '1'
+  stdout, stderror, retval = run_cmd(auth_user, PCS, *flags, "--", *cmd)
   return false if retval != 0
   return true
 end
 
 def disable_cluster(auth_user, all)
-  cmd = [PCS, "cluster", "disable"]
-  cmd << '--all' if all == '1'
-  stdout, stderror, retval = run_cmd(auth_user, *cmd)
+  cmd = ["cluster", "disable"]
+  flags = []
+  flags << '--all' if all == '1'
+  stdout, stderror, retval = run_cmd(auth_user, PCS, *flags, "--", *cmd)
   return false if retval != 0
   return true
 end
@@ -817,11 +794,15 @@ def get_corosync_version()
 end
 
 def pacemaker_running?()
-  is_service_running?('pacemaker')
+  return ServiceChecker.new(
+    ['pacemaker'], running: true
+  ).is_running?('pacemaker')
 end
 
 def pacemaker_remote_running?()
-  is_service_running?('pacemaker_remote')
+  return ServiceChecker.new(
+    ['pacemaker_remote'], running: true
+  ).is_running?('pacemaker_remote')
 end
 
 def get_pacemaker_version()
@@ -864,39 +845,48 @@ end
 def run_cmd_options(auth_user, options, *args)
   $logger.info("Running: " + args.join(" "))
   start = Time.now
-  out = ""
-  errout = ""
+  out = Tempfile.new
+  errout = Tempfile.new
 
-  proc_block = proc { |pid, stdin, stdout, stderr|
-    if options and options.key?('stdin')
-      stdin.puts(options['stdin'])
-      stdin.close()
-    end
-    out = stdout.readlines()
-    errout = stderr.readlines()
-    duration = Time.now - start
-    $logger.debug(out)
-    $logger.debug(errout)
-    $logger.debug("Duration: " + duration.to_s + "s")
-  }
   cib_user = auth_user[:username]
   # when running 'id -Gn' to get the groups they are not defined yet
   cib_groups = (auth_user[:usergroups] || []).join(' ')
   $logger.info("CIB USER: #{cib_user}, groups: #{cib_groups}")
-  # Open4.popen4 reimplementation which sets ENV in a child process prior
-  # to running an external process by exec
-  status = Open4::do_popen(proc_block, :init) { |ps_read, ps_write|
-    ps_read.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
-    ps_write.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
-    ENV['CIB_user'] = cib_user
-    ENV['CIB_user_groups'] = cib_groups
-    ENV['LC_ALL'] = 'C'
-    exec(*args)
-  }
 
-  retval = status.exitstatus
+  ChildProcess.posix_spawn = true
+  cmd = ChildProcess.build(*args)
+  cmd.io.stdout = out
+  cmd.io.stdout.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
+  cmd.io.stderr = errout
+  cmd.io.stderr.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
+  cmd.duplex = true
+  cmd.environment['CIB_user'] = cib_user
+  cmd.environment['CIB_user_groups'] = cib_groups
+  cmd.environment['LC_ALL'] = 'C'
+  cmd.start
+
+  if options and options.key?('stdin')
+    cmd.io.stdin.puts options['stdin']
+    cmd.io.stdin.close
+  end
+
+  cmd.wait
+  duration = Time.now - start
+  retval = cmd.exit_code
+
+  out.rewind
+  output = out.readlines
+  errout.rewind
+  error_output = errout.readlines
+  out.close
+  errout.close
+
+  $logger.debug(output.join(" "))
+  $logger.debug(error_output.join(" "))
+  $logger.debug("Duration: " + duration.to_s + "s")
   $logger.info("Return Value: " + retval.to_s)
-  return out, errout, retval
+
+  return output, error_output, retval
 end
 
 def is_score(score)
@@ -923,7 +913,8 @@ def is_auth_against_nodes(auth_user, node_names, timeout=10)
   offline_nodes = []
 
   node_names.uniq.each { |node_name|
-    threads << Thread.new {
+    threads << Thread.new(Thread.current[:pcsd_logger_container]) { |logger|
+      Thread.current[:pcsd_logger_container] = logger
       code, response = send_request_with_token(
         auth_user, node_name, 'check_auth', false, {}, true, nil, timeout
       )
@@ -947,6 +938,32 @@ def is_auth_against_nodes(auth_user, node_names, timeout=10)
   return online_nodes, offline_nodes, not_authorized_nodes
 end
 
+def pcs_auth_token(nodes)
+  # nodes is a hash of hashes:
+  # {
+  #   'node name' => {
+  #     'token' => auth token
+  #     'dest_list' => [ # currently only the first item is used
+  #       {'addr' => addr, 'port' => port} # how to connect to a node
+  #     ]
+  #   }
+  # }
+
+  new_hosts = []
+  nodes.each { |node_name, node_data|
+    new_hosts << PcsKnownHost.new(
+      node_name,
+      node_data['token'],
+      [node_data['dest_list'][0]]
+    )
+  }
+
+  # We want to avoid send updated known hosts to all cluster nodes therefore we
+  # are using 'nil' for cluster_name argument. Actually, automatic known hosts
+  # synchronization should be disabled in cluster when setting a custom token.
+  Cfgsync::save_sync_new_known_hosts(new_hosts, [], [], nil)
+end
+
 def pcs_auth(auth_user, nodes)
   # nodes is a hash of hashes:
   # {
@@ -963,7 +980,8 @@ def pcs_auth(auth_user, nodes)
   auth_responses = {}
   threads = []
   nodes.each { |node_name, node_data|
-    threads << Thread.new {
+    threads << Thread.new(Thread.current[:pcsd_logger_container]) { |logger|
+      Thread.current[:pcsd_logger_container] = logger
       begin
         addr = node_data.fetch('dest_list').fetch(0).fetch('addr')
         port = node_data.fetch('dest_list').fetch(0).fetch('port')
@@ -1103,10 +1121,7 @@ end
 def write_file_lock(path, perm, data, binary=false, user=nil, group=nil)
   file = nil
   begin
-    # File.open(path, mode, options)
-    # File.open(path, mode, perm, options)
-    # In order to set permissions, the method must be called with 4 arguments.
-    file = File.open(path, binary ? 'wb' : 'w', perm, {})
+    file = File.open(path, binary ? 'wb' : 'w', perm)
     file.flock(File::LOCK_EX)
     if user or group
       File.chown(get_uid(user), get_gid(group), path)
@@ -1140,39 +1155,23 @@ def read_file_lock(path, binary=false)
   end
 end
 
-def verify_cert_key_pair(cert, key)
+def verify_cert_key_pair(cert_data, key_data)
   errors = []
-  cert_modulus = nil
-  key_modulus = nil
 
-  stdout, stderr, retval = run_cmd_options(
-    PCSAuth.getSuperuserAuth(),
-    {
-      'stdin' => cert,
-    },
-    '/usr/bin/openssl', 'x509', '-modulus', '-noout'
-  )
-  if retval != 0
-    errors << "Invalid certificate: #{stderr.join}"
-  else
-    cert_modulus = stdout.join.strip
+  begin
+    cert = OpenSSL::X509::Certificate.new(cert_data)
+  rescue OpenSSL::X509::CertificateError => e
+    errors << "Invalid certificate: #{e}"
   end
 
-  stdout, stderr, retval = run_cmd_options(
-    PCSAuth.getSuperuserAuth(),
-    {
-      'stdin' => key,
-    },
-    '/usr/bin/openssl', 'rsa', '-modulus', '-noout'
-  )
-  if retval != 0
-    errors << "Invalid key: #{stderr.join}"
-  else
-    key_modulus = stdout.join.strip
+  begin
+    key = OpenSSL::PKey.read(key_data)
+  rescue OpenSSL::PKey::PKeyError => e
+    errors << "Invalid key: #{e}"
   end
 
-  if errors.empty? and cert_modulus and key_modulus
-    if cert_modulus != key_modulus
+  if errors.empty?
+    if not cert.check_private_key(key)
       errors << 'Certificate does not match the key'
     end
   end
@@ -1191,15 +1190,13 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     :status => 'unknown',
     :node_list => [],
     :resource_list => [],
-    # deprecated, kept for backward compatibility
-    # use pcsd_capabilities instead
-    :available_features => [],
     :pcsd_capabilities => [],
   }
 
   threads = []
   cluster_nodes.uniq.each { |node|
-    threads << Thread.new {
+    threads << Thread.new(Thread.current[:pcsd_logger_container]) { |logger|
+      Thread.current[:pcsd_logger_container] = logger
       code, response = send_request_with_token(
         auth_user,
         node,
@@ -1223,7 +1220,6 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
       }
       begin
         parsed_response = JSON.parse(response, {:symbolize_names => true})
-        parsed_response[:available_features] ||= []
         parsed_response[:pcsd_capabilities] ||= []
         if parsed_response[:noresponse]
           node_map[node][:node] = {}
@@ -1236,10 +1232,11 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
           if parsed_response[:node]
             parsed_response[:status_version] = '2'
             parsed_response[:node][:status_version] = '2'
+            node_map[node] = parsed_response
           else
-            parsed_response = status_v1_to_v2(parsed_response)
+            node_map[node][:node] = {}
+            node_map[node][:node].update(node_status_unknown)
           end
-          node_map[node] = parsed_response
         end
         node_map[node][:node][:name] = node
       rescue JSON::ParserError
@@ -1254,7 +1251,6 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
   node_status_list = []
   quorate_nodes = []
   not_authorized_nodes = []
-  old_status = false
   node_map.each { |node_name, cluster_status|
     # If we were able to get node's cluster name and it's different than
     # requested cluster name, the node belongs to some other cluster and its
@@ -1264,7 +1260,6 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     next if cluster_status[:cluster_name] != cluster_name
     cluster_nodes_map[node_name] = cluster_status
     node_status_list << cluster_status[:node]
-    old_status = true if '1' == cluster_status[:status_version]
     quorate_nodes << node_name if cluster_status[:node][:quorum]
     not_authorized_nodes << node_name if cluster_status[:node][:notauthorized]
   }
@@ -1284,7 +1279,7 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
   # if we don't have quorum, use data from any online node,
   # otherwise use data from any node no node has quorum, so no node has any
   # info about the cluster
-  elsif not old_status
+  else
     node_to_use = cluster_nodes_map.values[0]
     cluster_nodes_map.each { |_, node_data|
       if node_data[:node] and node_data[:node][:status] == 'online'
@@ -1295,18 +1290,6 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     status = overview.update(node_to_use)
     status[:quorate] = false
     status[:node_list] = node_status_list
-  # old pcsd doesn't provide info about quorum, use data from any node
-  else
-    status = overview
-    status[:quorate] = nil
-    status[:node_list] = node_status_list
-    cluster_nodes_map.each { |_, node|
-      if node[:status_version] and node[:status_version] == '1' and
-          !node[:cluster_settings][:error]
-        status = overview.update(node)
-        break
-      end
-    }
   end
   status.delete(:node)
   sbd_enabled = []
@@ -1316,11 +1299,7 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
     node_status = cluster_status[:node][:status]
     node_name = cluster_status[:node][:name]
     # Create a set of available features on all nodes as an intersection of
-    # available features from all nodes. Do it for both the old deprecated list
-    # (available_features) and the new one (pcsd_capabilities)
-    if node_status != 'unknown' and cluster_status[:available_features]
-      status[:available_features] &= cluster_status[:available_features]
-    end
+    # available features from all nodes.
     if node_status != 'unknown' and cluster_status[:pcsd_capabilities]
       status[:pcsd_capabilities] &= cluster_status[:pcsd_capabilities]
     end
@@ -1394,17 +1373,9 @@ def cluster_status_from_nodes(auth_user, cluster_nodes, cluster_name)
   end
 
   if status[:quorate].nil?
-    if old_status
-      status[:warning_list] << {
-        :message => 'Cluster is running an old version of pcs/pcsd which '\
-          + "doesn't provide data for the dashboard.",
-        :type => 'old_pcsd'
-      }
-    else
-      status[:error_list] << {
-        :message => 'Unable to connect to the cluster.'
-      }
-    end
+    status[:error_list] << {
+      :message => 'Unable to connect to the cluster.'
+    }
     status[:status] = 'unknown'
     return status
   end
@@ -1450,6 +1421,7 @@ end
 def get_node_status(auth_user, cib_dom)
   node_status = {
       :cluster_name => $cluster_name,
+      :cluster_uuid => $cluster_uuid,
       :groups => [],
       :constraints => {
           # :rsc_location => [],
@@ -1460,21 +1432,10 @@ def get_node_status(auth_user, cib_dom)
       :acls => get_acls(auth_user, cib_dom),
       :username => auth_user[:username],
       :fence_levels => get_fence_levels(auth_user, cib_dom),
-      :node_attr => node_attrs_to_v2(get_node_attributes(auth_user, cib_dom)),
+      :node_attr => get_node_attributes(auth_user, cib_dom),
       :nodes_utilization => get_nodes_utilization(cib_dom),
       :alerts => get_alerts(auth_user),
       :known_nodes => [],
-      # deprecated, kept for backward compatibility
-      # use pcsd_capabilities instead
-      :available_features => [
-        'constraint_colocation_set',
-        'sbd',
-        'ticket_constraints',
-        'moving_resource_in_group',
-        'unmanaged_resource',
-        'alerts',
-        'hardened_cluster',
-      ],
       :pcsd_capabilities => CAPABILITIES_PCSD
   }
 
@@ -1539,42 +1500,9 @@ def get_resources(cib_dom, crm_dom=nil, get_operations=false)
   return resource_list
 end
 
-def get_resource_by_id(id, cib_dom, crm_dom=nil, rsc_status=nil, operations=false)
-  unless cib_dom
-    return nil
-  end
-
-  e = cib_dom.elements["/cib/configuration/resources//*[@id='#{id}']"]
-  unless e
-    return nil
-  end
-
-  if e.parent.name != 'resources' # if resource is in group, clone or master/slave
-    p = get_resource_by_id(
-      e.parent.attributes['id'], cib_dom, crm_dom, rsc_status, operations
-    )
-    return p.get_map[id.to_sym]
-  end
-
-  case e.name
-    when 'primitive'
-      return ClusterEntity::Primitive.new(e, rsc_status, nil, operations)
-    when 'group'
-      return ClusterEntity::Group.new(e, rsc_status, nil, operations)
-    when 'clone'
-      return ClusterEntity::Clone.new(e, crm_dom, rsc_status, nil, operations)
-    when 'master'
-      return ClusterEntity::Clone.new(e, crm_dom, rsc_status, nil, operations)
-    else
-      return nil
-  end
-end
-
 def get_crm_mon_dom(auth_user)
   begin
-    stdout, _, retval = run_cmd(
-      auth_user, CRM_MON, '--one-shot', '-r', '--as-xml'
-    )
+    stdout, _, retval = run_crm_mon_xml(auth_user)
     if retval == 0
       return REXML::Document.new(stdout.join("\n"))
     end
@@ -1596,79 +1524,6 @@ def get_cib_dom(auth_user)
   return nil
 end
 
-def node_attrs_to_v2(node_attrs)
-  all_nodes_attr = {}
-  node_attrs.each { |node, attrs|
-    all_nodes_attr[node] = []
-    attrs.each { |attr|
-      all_nodes_attr[node] << {
-        :id => attr[:id],
-        :name => attr[:key],
-        :value => attr[:value]
-      }
-    }
-  }
-  return all_nodes_attr
-end
-
-def status_v1_to_v2(status)
-  new_status = status.select { |k,_|
-    [:cluster_name, :username,
-     :need_ring1_address, :cluster_settings, :constraints,
-     :corosync_online, :corosync_offline, :pacemaker_online, :pacemaker_standby,
-     :pacemaker_offline, :acls, :fence_levels
-    ].include?(k)
-  }
-  new_status[:node_attr] = node_attrs_to_v2(status[:node_attr])
-
-  resources = ClusterEntity::make_resources_tree(
-    ClusterEntity::get_primitives_from_status_v1(status[:resources])
-  )
-  resources_hash = []
-  resources.each { |r|
-    resources_hash << r.to_status('2')
-  }
-  new_status[:resource_list] = resources_hash
-  new_status[:node] = status.select { |k,_|
-    [:uptime, :corosync, :pacemaker, :corosync_enabled,
-     :pacemaker_enabled, :pcsd_enabled
-    ].include?(k)
-  }
-
-  new_status[:groups] = get_group_list_from_tree_of_resources(resources)
-
-  new_status[:node].update(
-    {
-      :id => status[:node_id],
-      :quorum => nil,
-      :warning_list => [],
-      :error_list => [],
-      :status => (new_status[:node][:corosync] and
-        new_status[:node][:pacemaker]) ? "online" : "offline",
-      :status_version => '1'
-    }
-  )
-  new_status[:status_version] = '1'
-
-  return new_status
-end
-
-def get_group_list_from_tree_of_resources(tree)
-  group_list = []
-  tree.each { |resource|
-    if resource.instance_of?(ClusterEntity::Group)
-      group_list << resource.id
-    end
-    if (
-      resource.kind_of?(ClusterEntity::MultiInstancePcmk1) and
-      resource.member.instance_of?(ClusterEntity::Group)
-    )
-      group_list << resource.member.id
-    end
-  }
-  return group_list
-end
-
 def allowed_for_local_cluster(auth_user, action)
   pcs_config = PCSConfig.new(Cfgsync::PcsdSettings.from_file().text())
   return pcs_config.permissions_local.allows?(
@@ -1688,175 +1543,117 @@ def allowed_for_superuser(auth_user)
   return true
 end
 
-def get_default_overview_node_list(clustername)
-  nodes = get_cluster_nodes clustername
-  node_list = []
-  nodes.each { |node|
-    node_list << {
-      'error_list' => [],
-      'warning_list' => [],
-      'status' => 'unknown',
-      'quorum' => false,
-      'name' => node
-    }
-  }
-  return node_list
-end
-
-def is_service_enabled?(service)
-  if ISSYSTEMCTL
-    cmd = ['systemctl', 'is-enabled', "#{service}.service"]
-  else
-    cmd = ['chkconfig', service]
-  end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
-end
-
-def is_service_running?(service)
-  if ISSYSTEMCTL
-    # --lines=0 disables listing last N lines from journal related to the
-    # service. The lines may contain non-ASCII characters which cause various
-    # encoding/decoding errors. We are not interested in the lines anyway.
-    cmd = ['systemctl', 'status', '--lines=0', "#{service}.service"]
-  else
-    cmd = ['service', service, 'status']
-  end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
-end
-
-class ServiceInstalledChecker
-  def initialize()
-    @list_unit_files_output = self.load_unit_files_list()
-  end
-
-  def is_installed?(service)
-    if @list_unit_files_output.nil?
-      return nil
-    end
-
-    @list_unit_files_output.each { |line|
-      if self.contains_line_service?(line, service)
-        return true
-      end
-    }
-    return false
-  end
-
-  protected
-  def load_unit_files_list()
-    stdout, _, retcode = self.run_command()
-    if retcode != 0
-      return nil
-    end
-    return stdout
-  end
-end
-
-class ServiceInstalledCheckerSystemctl < ServiceInstalledChecker
-  protected
-  def run_command
-    # currently we are not using systemd instances (service_name@instance) in pcsd
-    # for proper implementation of is_service_installed see
-    # pcs/lib/external.py:is_service_installed
-    return run_cmd(
-      PCSAuth.getSuperuserAuth(), 'systemctl', 'list-unit-files', '--full'
-    )
-  end
-
-  def contains_line_service?(line, service)
-    return line.strip().start_with?("#{service}.service")
-  end
-end
-
-class ServiceInstalledCheckerChkconfig < ServiceInstalledChecker
-  protected
-  def run_command
-    return run_cmd(PCSAuth.getSuperuserAuth(), 'chkconfig')
-  end
-
-  def contains_line_service?(line, service)
-    return line.split(' ')[0] == service
-  end
-end
-
-def get_service_installed_checker
-  if ISSYSTEMCTL
-    return ServiceInstalledCheckerSystemctl.new
-  else
-    return ServiceInstalledCheckerChkconfig.new
-  end
-end
-
-
-def is_service_installed?(service)
-  return get_service_installed_checker().is_installed?(service)
-end
-
 def enable_service(service)
-  if ISSYSTEMCTL
-    # fails when the service is not installed
-    cmd = ['systemctl', 'enable', "#{service}.service"]
-  else
-    # fails when the service is not installed
-    cmd = ['chkconfig', service, 'on']
-  end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
+  result = run_pcs_internal(
+    PCSAuth.getSuperuserAuth(),
+    "services.enable_service",
+    {:service => service, :instance => nil},
+  )
+  return result[:status] == 'success'
 end
 
 def disable_service(service)
-  # fails when the service is not installed, so we need to check it beforehand
-  if not is_service_installed?(service)
-    return true
-  end
-
-  if ISSYSTEMCTL
-    cmd = ['systemctl', 'disable', "#{service}.service"]
-  else
-    cmd = ['chkconfig', service, 'off']
-  end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
+  result = run_pcs_internal(
+    PCSAuth.getSuperuserAuth(),
+    "services.disable_service",
+    {:service => service, :instance => nil},
+  )
+  return result[:status] == 'success'
 end
 
 def start_service(service)
-  if ISSYSTEMCTL
-    cmd = ['systemctl', 'start', "#{service}.service"]
-  else
-    cmd = ['service', service, 'start']
-  end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
+  result = run_pcs_internal(
+    PCSAuth.getSuperuserAuth(),
+    "services.start_service",
+    {:service => service, :instance => nil},
+  )
+  return result[:status] == 'success'
 end
 
 def stop_service(service)
-  if not is_service_installed?(service)
+  if not ServiceChecker.new([service], installed: true).is_installed?(service)
     return true
   end
-  if ISSYSTEMCTL
-    cmd = ['systemctl', 'stop', "#{service}.service"]
-  else
-    cmd = ['service', service, 'stop']
+  result = run_pcs_internal(
+    PCSAuth.getSuperuserAuth(),
+    "services.stop_service",
+    {:service => service, :instance => nil},
+  )
+  return result[:status] == 'success'
+end
+
+class ServiceChecker
+  def initialize(services, flags={})
+    result = run_pcs_internal(
+      PCSAuth.getSuperuserAuth(),
+      "services.get_services_info",
+      {
+        :services => services,
+        :installed => flags.fetch(:installed, true),
+        :enabled => flags.fetch(:enabled, false),
+        :running => flags.fetch(:running, false),
+      },
+    )
+    @services_info = {}
+    if result[:status] == 'success'
+      data = result[:data]
+      unless data.nil?
+        @services_info = Hash[
+          data[:services].collect{ |service| [service[:service], service]}
+        ]
+      end
+    end
+  end
+
+  def get_service(service)
+    info = @services_info[service]
+    if info.nil?
+      return {
+        :service => service,
+        :installed => nil,
+        :enabled => nil,
+        :running => nil,
+      }
+    end
+    return info
+  end
+
+  def get_info(service)
+    return {
+      :installed => self.is_installed?(service),
+      :enabled => self.is_enabled?(service),
+      :running => self.is_running?(service),
+    }
+  end
+
+  def is_installed?(service)
+    return !!self.get_service(service)[:installed]
+  end
+
+  def is_enabled?(service)
+    return !!self.get_service(service)[:enabled]
+  end
+
+  def is_running?(service)
+    return !!self.get_service(service)[:running]
   end
-  _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
-  return (retcode == 0)
 end
 
 def set_cluster_prop_force(auth_user, prop, val)
-  cmd = [PCS, 'property', 'set', "#{prop}=#{val}", '--force']
+  cmd = ['property', 'set', "#{prop}=#{val}"]
+  flags = ['--force']
   if pacemaker_running?
-    _, _, retcode = run_cmd(auth_user, *cmd)
+    user = auth_user
   else
-    cmd += ['-f', CIB_PATH]
-    _, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
+    user = PCSAuth.getSuperuserAuth()
+    flags += ['-f', CIB_PATH]
   end
+    _, _, retcode = run_cmd(user, PCS, *flags, "--", *cmd)
   return (retcode == 0)
 end
 
 def get_parsed_local_sbd_config()
-  cmd = [PCS, 'stonith', 'sbd', 'local_config_in_json']
+  cmd = [PCS, '--', 'stonith', 'sbd', 'local_config_in_json']
   out, _, retcode = run_cmd(PCSAuth.getSuperuserAuth(), *cmd)
   if retcode != 0
     return nil
@@ -1910,7 +1707,7 @@ def get_authfile_from_booth_config(config_data)
 end
 
 def get_alerts(auth_user)
-  out, _, retcode = run_cmd(auth_user, PCS, 'alert', 'get_all_alerts')
+  out, _, retcode = run_cmd(auth_user, PCS, '--', 'alert', 'get_all_alerts')
 
   if retcode !=  0
     return nil
@@ -1923,15 +1720,6 @@ def get_alerts(auth_user)
   end
 end
 
-def get_service_info(service_name, service_checker)
-  return {
-    :installed => service_checker.is_installed?(service_name),
-    :enabled => is_service_enabled?(service_name),
-    :running => is_service_running?(service_name),
-    :version => nil,
-  }
-end
-
 def get_pcs_internal_output_format(status, status_msg=nil)
   return {
     :status => status,
@@ -1941,18 +1729,6 @@ def get_pcs_internal_output_format(status, status_msg=nil)
   }
 end
 
-def pcs_internal_proxy(auth_user, data, cmd)
-  begin
-    input_data = JSON.parse(data)
-    return JSON.generate(run_pcs_internal(auth_user, cmd, input_data))
-  rescue JSON::ParserError => e
-    $logger.error("Invalid input data format: #{e}")
-    return JSON.generate(get_pcs_internal_output_format(
-      'input_error', "Invalid input data format: #{e}"
-    ))
-  end
-end
-
 def run_pcs_internal(auth_user, cmd, data, request_timeout=nil)
   input_data = {
     :cmd => cmd,
@@ -1972,15 +1748,15 @@ def run_pcs_internal(auth_user, cmd, data, request_timeout=nil)
     )
   end
   begin
-    parsed_output = JSON.parse(stdout.join("\n"))
+    parsed_output = JSON.parse(stdout.join("\n"), {:symbolize_names => true})
     if (
-      parsed_output.include?('report_list') \
+      parsed_output.include?(:report_list) \
       and \
-      parsed_output['report_list'].kind_of?(Array) \
+      parsed_output[:report_list].kind_of?(Array) \
     )
-      # Remove all debug messages as they may containt sensitive info.
-      parsed_output['report_list'].delete_if { |report_item|
-        report_item["severity"] == "DEBUG"
+      # Remove all debug messages as they may contain sensitive info.
+      parsed_output[:report_list].delete_if { |report_item|
+        report_item[:severity][:level] == 'DEBUG'
       }
     end
     return parsed_output
